@@ -27,9 +27,9 @@ static const int MAX_DPX_IMAGE_ELEMENTS = 8;  // max subimages in DPX spec
 class DPXOutput final : public ImageOutput {
 public:
     DPXOutput();
-    virtual ~DPXOutput();
-    virtual const char* format_name(void) const override { return "dpx"; }
-    virtual int supports(string_view feature) const override
+    ~DPXOutput() override;
+    const char* format_name(void) const override { return "dpx"; }
+    int supports(string_view feature) const override
     {
         if (feature == "multiimage" || feature == "alpha"
             || feature == "nchannels" || feature == "random_access"
@@ -38,21 +38,16 @@ public:
             return true;
         return false;
     }
-    virtual bool open(const std::string& name, const ImageSpec& spec,
-                      OpenMode mode = Create) override;
-    virtual bool open(const std::string& name, int subimages,
-                      const ImageSpec* specs) override;
-    virtual bool close() override;
-    virtual bool write_scanline(int y, int z, TypeDesc format, const void* data,
-                                stride_t xstride) override;
-    virtual bool write_tile(int x, int y, int z, TypeDesc format,
-                            const void* data, stride_t xstride,
-                            stride_t ystride, stride_t zstride) override;
-    virtual bool set_ioproxy(Filesystem::IOProxy* ioproxy) override
-    {
-        m_io = ioproxy;
-        return true;
-    }
+    bool open(const std::string& name, const ImageSpec& spec,
+              OpenMode mode = Create) override;
+    bool open(const std::string& name, int subimages,
+              const ImageSpec* specs) override;
+    bool close() override;
+    bool write_scanline(int y, int z, TypeDesc format, const void* data,
+                        stride_t xstride) override;
+    bool write_tile(int x, int y, int z, TypeDesc format, const void* data,
+                    stride_t xstride, stride_t ystride,
+                    stride_t zstride) override;
 
 private:
     OutStream* m_stream = nullptr;
@@ -74,8 +69,6 @@ private:
     bool m_write_pending;  // subimage buffer needs to be written
     unsigned int m_dither;
     std::vector<unsigned char> m_tilebuffer;
-    std::unique_ptr<Filesystem::IOProxy> m_io_local;
-    Filesystem::IOProxy* m_io = nullptr;
 
     // Initialize private members to pre-opened state
     void init(void)
@@ -90,8 +83,7 @@ private:
         m_subimages_to_write = 0;
         m_subimage_specs.clear();
         m_write_pending = false;
-        m_io_local.reset();
-        m_io = nullptr;
+        ioproxy_clear();
     }
 
     // Is the output file currently opened?
@@ -117,7 +109,7 @@ private:
 
 
 
-// Obligatory material to make this a recognizeable imageio plugin:
+// Obligatory material to make this a recognizable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
 OIIO_EXPORT ImageOutput*
@@ -196,23 +188,11 @@ DPXOutput::open(const std::string& name, const ImageSpec& userspec,
     if (is_opened())
         close();  // Close any already-opened file
 
-    const ParamValue* param = userspec.find_attribute("oiio:ioproxy",
-                                                      TypeDesc::PTR);
-    if (param)
-        m_io = param->get<Filesystem::IOProxy*>();
-
-    if (!m_io) {
-        // If no proxy was supplied, create a file reader
-        m_io = new Filesystem::IOFile(name, Filesystem::IOProxy::Mode::Write);
-        m_io_local.reset(m_io);
-    }
-
-    if (!m_io || m_io->mode() != Filesystem::IOProxy::Mode::Write) {
-        errorf("Could not open file \"%s\"", name);
+    ioproxy_retrieve_from_config(userspec);
+    if (!ioproxy_use_or_open(name))
         return false;
-    }
 
-    m_stream = new OutStream(m_io);
+    m_stream = new OutStream(ioproxy());
     m_dpx.SetOutStream(m_stream);
     m_dpx.Start();
     m_subimage = 0;
@@ -467,7 +447,7 @@ DPXOutput::prep_subimage(int s, bool allocate)
     std::string colorspace = m_spec.get_string_attribute("oiio:ColorSpace", "");
     if (Strutil::iequals(colorspace, "Linear"))
         m_transfer = dpx::kLinear;
-    else if (Strutil::iequals(colorspace, "GammaCorrected"))
+    else if (Strutil::istarts_with(colorspace, "Gamma"))
         m_transfer = dpx::kUserDefined;
     else if (Strutil::iequals(colorspace, "Rec709"))
         m_transfer = dpx::kITUR709;
@@ -739,42 +719,32 @@ DPXOutput::set_keycode_values(int* array)
 {
     // Manufacturer code
     {
-        std::stringstream ss;
-        ss << std::setfill('0');
-        ss << std::setw(2) << array[0];
-        memcpy(m_dpx.header.filmManufacturingIdCode, ss.str().c_str(), 2);
+        std::string ss = Strutil::fmt::format("{:02d}", array[0]);
+        memcpy(m_dpx.header.filmManufacturingIdCode, ss.c_str(), 2);
     }
 
     // Film type
     {
-        std::stringstream ss;
-        ss << std::setfill('0');
-        ss << std::setw(2) << array[1];
-        memcpy(m_dpx.header.filmType, ss.str().c_str(), 2);
+        std::string ss = Strutil::fmt::format("{:02d}", array[1]);
+        memcpy(m_dpx.header.filmType, ss.c_str(), 2);
     }
 
     // Prefix
     {
-        std::stringstream ss;
-        ss << std::setfill('0');
-        ss << std::setw(6) << array[2];
-        memcpy(m_dpx.header.prefix, ss.str().c_str(), 6);
+        std::string ss = Strutil::fmt::format("{:06d}", array[2]);
+        memcpy(m_dpx.header.prefix, ss.c_str(), 6);
     }
 
     // Count
     {
-        std::stringstream ss;
-        ss << std::setfill('0');
-        ss << std::setw(4) << array[3];
-        memcpy(m_dpx.header.count, ss.str().c_str(), 4);
+        std::string ss = Strutil::fmt::format("{:04d}", array[3]);
+        memcpy(m_dpx.header.count, ss.c_str(), 4);
     }
 
     // Perforation Offset
     {
-        std::stringstream ss;
-        ss << std::setfill('0');
-        ss << std::setw(2) << array[4];
-        memcpy(m_dpx.header.perfsOffset, ss.str().c_str(), 2);
+        std::string ss = Strutil::fmt::format("{:02d}", array[4]);
+        memcpy(m_dpx.header.perfsOffset, ss.c_str(), 2);
     }
 
     // Format
