@@ -37,7 +37,6 @@
 #include <typeinfo>
 #include <type_traits>
 
-#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/span.h>
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/oiioversion.h>
@@ -47,6 +46,15 @@
 
 OIIO_NAMESPACE_BEGIN
 
+/// If the caller defines OIIO_FMATH_HEADER_ONLY to nonzero, then 100% of the
+/// implementation of fmath functions will be defined directly in this header
+/// file.  If it is not defined, or set to 0, then there are a few functions
+/// for which this header will only provide the definition.
+#ifndef OIIO_FMATH_HEADER_ONLY
+#    define OIIO_FMATH_HEADER_ONLY 0
+#endif
+
+
 /// Occasionally there may be a tradeoff where the best/fastest
 /// implementation of a math function in an ordinary scalar context does
 /// things that prevent autovectorization (or OMP pragma vectorization) of a
@@ -341,7 +349,7 @@ clamp (const T& a, const T& low, const T& high)
 {
 #if 1
     // This looks clunky, but it generates minimal code. For float, it
-    // should result in just a max and min instruction, thats it.
+    // should result in just a max and min instruction, that's it.
     // This implementation is courtesy of Alex Wells, Intel, via OSL.
     T val = a;
     if (!(low <= val))  // Forces clamp(NaN,low,high) to return low
@@ -402,7 +410,7 @@ clamp (const simd::vint16& a, const simd::vint16& low, const simd::vint16& high)
 
 
 
-// For the multply+add (or sub) operations below, note that the results may
+// For the multiply+add (or sub) operations below, note that the results may
 // differ slightly on different hardware, depending on whether true fused
 // multiply and add is available or if the code generated just does an old
 // fashioned multiply followed by a separate add. So please interpret these
@@ -749,9 +757,12 @@ inline OIIO_HOSTDEVICE float sign (float x)
 //
 // Type and range conversion helper functions and classes.
 
-
-template <typename IN_TYPE, typename OUT_TYPE>
-OIIO_FORCEINLINE OIIO_HOSTDEVICE OUT_TYPE bit_cast (const IN_TYPE& in) {
+/// Standards-compliant bit cast of two equally sized types. This is used
+/// equivalently to C++20 std::bit_cast, but it works prior to C++20 and
+/// it has the right decorators to work with Cuda.
+/// @version 2.4.1
+template <typename OUT_TYPE, typename IN_TYPE>
+OIIO_FORCEINLINE OIIO_HOSTDEVICE OUT_TYPE bitcast (const IN_TYPE& in) noexcept {
     // NOTE: this is the only standards compliant way of doing this type of casting,
     // luckily the compilers we care about know how to optimize away this idiom.
     static_assert(sizeof(IN_TYPE) == sizeof(OUT_TYPE),
@@ -761,45 +772,64 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE OUT_TYPE bit_cast (const IN_TYPE& in) {
     return out;
 }
 
-#if defined(__INTEL_COMPILER)
-    // On x86/x86_64 for certain compilers we can use Intel CPU intrinsics
-    // for some common bit_cast cases that might be even more understandable
-    // to the compiler and generate better code without its getting confused
-    // about the memcpy in the general case.
-    // FIXME: The intrinsics are not in clang <= 9 nor gcc <= 9.1. Check
-    // future releases.
-    template<> OIIO_FORCEINLINE uint32_t bit_cast<float, uint32_t>(const float& val) {
-          return static_cast<uint32_t>(_castf32_u32(val));
-    }
-    template<> OIIO_FORCEINLINE int32_t bit_cast<float, int32_t>(const float& val) {
-          return static_cast<int32_t>(_castf32_u32(val));
-    }
-    template<> OIIO_FORCEINLINE float bit_cast<uint32_t, float>(const uint32_t& val) {
-          return _castu32_f32(val);
-    }
-    template<> OIIO_FORCEINLINE float bit_cast<int32_t, float>(const int32_t& val) {
-          return _castu32_f32(val);
-    }
-    template<> OIIO_FORCEINLINE uint64_t bit_cast<double, uint64_t>(const double& val) {
-          return static_cast<uint64_t>(_castf64_u64(val));
-    }
-    template<> OIIO_FORCEINLINE int64_t bit_cast<double, int64_t>(const double& val) {
-          return static_cast<int64_t>(_castf64_u64(val));
-    }
-    template<> OIIO_FORCEINLINE double bit_cast<uint64_t, double>(const uint64_t& val) {
-          return _castu64_f64(val);
-    }
-    template<> OIIO_FORCEINLINE double bit_cast<int64_t, double>(const int64_t& val) {
-          return _castu64_f64(val);
-    }
+#if OIIO_VERSION_LESS(3, 0, 0)
+/// Note: The C++20 std::bit_cast has the reverse order of the template
+/// arguments of our original bit_cast! That is unfortunate. For now, we
+/// prefer using OIIO::bitcast. We'll keep this old one for backward
+/// compatibility, but will eventually deprecate for OIIO 2.5 and remove it
+/// for 3.0.
+template <typename IN_TYPE, typename OUT_TYPE>
+#if OIIO_VERSION_GREATER_EQUAL(2, 5, 0)
+OIIO_DEPRECATED("Use OIIO::bitcast<To, From> instead")
+#endif
+OIIO_FORCEINLINE OIIO_HOSTDEVICE OUT_TYPE bit_cast (const IN_TYPE& in) {
+    return bitcast<OUT_TYPE, IN_TYPE>(in);
+}
+#endif
+
+#if defined(__x86_64__) && !defined(__CUDA_ARCH__) && \
+    (defined(__INTEL_COMPILER) || defined(__INTEL_LLVM_COMPILER) \
+     || OIIO_CLANG_VERSION >= 100000 || OIIO_APPLE_CLANG_VERSION >= 130000 \
+     || OIIO_GCC_VERSION >= 90300)
+// On x86/x86_64 for certain compilers we can use Intel CPU intrinsics for
+// some common bitcast cases that might be even more understandable to the
+// compiler and generate better code without its getting confused about the
+// memcpy in the general case. We're a bit conservative with the compiler
+// version checks here, it may be that some earlier versions support these
+// intrinsics.
+
+template<> OIIO_FORCEINLINE uint32_t bitcast<uint32_t, float>(const float& val) noexcept {
+    return static_cast<uint32_t>(_castf32_u32(val));
+}
+template<> OIIO_FORCEINLINE int32_t bitcast<int32_t, float>(const float& val) noexcept {
+    return static_cast<int32_t>(_castf32_u32(val));
+}
+template<> OIIO_FORCEINLINE float bitcast<float, uint32_t>(const uint32_t& val) noexcept {
+    return _castu32_f32(val);
+}
+template<> OIIO_FORCEINLINE float bitcast<float, int32_t>(const int32_t& val) noexcept {
+    return _castu32_f32(val);
+}
+template<> OIIO_FORCEINLINE uint64_t bitcast<uint64_t, double>(const double& val) noexcept {
+    return static_cast<uint64_t>(_castf64_u64(val));
+}
+template<> OIIO_FORCEINLINE int64_t bitcast<int64_t, double>(const double& val) noexcept {
+    return static_cast<int64_t>(_castf64_u64(val));
+}
+template<> OIIO_FORCEINLINE double bitcast<double, uint64_t>(const uint64_t& val) noexcept {
+    return _castu64_f64(val);
+}
+template<> OIIO_FORCEINLINE double bitcast<double, int64_t>(const int64_t& val) noexcept {
+    return _castu64_f64(val);
+}
 #endif
 
 
 OIIO_FORCEINLINE OIIO_HOSTDEVICE int bitcast_to_int (float x) {
-    return bit_cast<float,int>(x);
+    return bitcast<int, float>(x);
 }
 OIIO_FORCEINLINE OIIO_HOSTDEVICE float bitcast_to_float (int x) {
-    return bit_cast<int,float>(x);
+    return bitcast<float, int>(x);
 }
 
 
@@ -827,7 +857,7 @@ swap_endian (T *f, int len=1)
 }
 
 
-#if (OIIO_GNUC_VERSION || OIIO_CLANG_VERSION || OIIO_APPLE_CLANG_VERSION || OIIO_INTEL_COMPILER_VERSION) && !defined(__CUDACC__)
+#if (OIIO_GNUC_VERSION || OIIO_ANY_CLANG || OIIO_INTEL_CLASSIC_COMPILER_VERSION) && !defined(__CUDACC__)
 // CPU gcc and compatible can use these intrinsics, 8-15x faster
 
 template<> inline void swap_endian(uint16_t* f, int len) {
@@ -922,7 +952,7 @@ inline OIIO_HOSTDEVICE D
 scaled_conversion(const S& src, F scale, F min, F max)
 {
     if (std::numeric_limits<S>::is_signed) {
-        F s = src * scale;
+        F s = src * scale;  //NOSONAR
         s += (s < 0 ? (F)-0.5 : (F)0.5);
         return (D)clamp(s, min, max);
     } else {
@@ -1042,32 +1072,6 @@ inline void convert_type<uint16_t,float> (const uint16_t *src,
 }
 
 
-#if defined(_HALF_H_) || defined(IMATH_HALF_H_)
-template<>
-inline void convert_type<half,float> (const half *src,
-                                      float *dst, size_t n,
-                                      float /*_min*/, float /*_max*/)
-{
-#if OIIO_SIMD >= 8 && OIIO_F16C_ENABLED
-    // If f16c ops are enabled, it's worth doing this by 8's
-    for ( ; n >= 8; n -= 8, src += 8, dst += 8) {
-        simd::vfloat8 s_simd (src);
-        s_simd.store (dst);
-    }
-#endif
-#if OIIO_SIMD >= 4
-    for ( ; n >= 4; n -= 4, src += 4, dst += 4) {
-        simd::vfloat4 s_simd (src);
-        s_simd.store (dst);
-    }
-#endif
-    while (n--)
-        *dst++ = (*src++);
-}
-#endif
-
-
-
 template<>
 inline void
 convert_type<float,uint16_t> (const float *src, uint16_t *dst, size_t n,
@@ -1112,7 +1116,39 @@ convert_type<float,uint8_t> (const float *src, uint8_t *dst, size_t n,
 
 #if defined(_HALF_H_) || defined(IMATH_HALF_H_)
 template<>
-inline void
+OIIO_UTIL_API
+void convert_type<half,float> (const half *src, float *dst, size_t n,
+                               float /*_min*/, float /*_max*/);
+template<>
+OIIO_UTIL_API
+void convert_type<float,half> (const float *src, half *dst, size_t n,
+                               half /*_min*/, half /*_max*/);
+
+#if OIIO_FMATH_HEADER_ONLY
+// Not just the declarations, give the definitions here.
+template<>
+void convert_type<half,float> (const half *src, float *dst, size_t n,
+                               float /*_min*/, float /*_max*/)
+{
+#if OIIO_SIMD >= 8 && OIIO_F16C_ENABLED
+    // If f16c ops are enabled, it's worth doing this by 8's
+    for ( ; n >= 8; n -= 8, src += 8, dst += 8) {
+        simd::vfloat8 s_simd (src);
+        s_simd.store (dst);
+    }
+#endif
+#if OIIO_SIMD >= 4
+    for ( ; n >= 4; n -= 4, src += 4, dst += 4) {
+        simd::vfloat4 s_simd (src);
+        s_simd.store (dst);
+    }
+#endif
+    while (n--)
+        *dst++ = (*src++);
+}
+
+template<>
+void
 convert_type<float,half> (const float *src, half *dst, size_t n,
                           half /*_min*/, half /*_max*/)
 {
@@ -1132,8 +1168,10 @@ convert_type<float,half> (const float *src, half *dst, size_t n,
     while (n--)
         *dst++ = *src++;
 }
-#endif
-#endif
+#endif /* if OIIO_FMATH_HEADER_ONLY */
+#endif /* if defined(IMATH_HALF_H_) */
+
+#endif /* ifndef __CUDA_ARCH__ */
 
 
 
@@ -1216,7 +1254,7 @@ bit_range_convert(unsigned int in, unsigned int FROM_BITS, unsigned int TO_BITS)
 
 
 
-/// Append the `n` LSB bits of `val` into a bit sting `T out[]`, where the
+/// Append the `n` LSB bits of `val` into a bit string `T out[]`, where the
 /// `filled` MSB bits of `*out` are already filled in. Increment `out` and
 /// adjust `filled` as required. Type `T` should be uint8_t, uint16_t, or
 /// uint32_t.
@@ -1633,7 +1671,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE int fast_rint (float x) {
     // single roundps instruction on SSE4.1+ (for gcc/clang at least)
     return static_cast<int>(std::rint(x));
 #else
-    // emulate rounding by adding/substracting 0.5
+    // emulate rounding by adding/subtracting 0.5
     return static_cast<int>(x + copysignf(0.5f, x));
 #endif
 }
@@ -1893,7 +1931,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE float fast_atan2 (float y, float x) {
     if (b_is_greater_than_a)
         r = 1.570796326794896557998982f - r; // account for arg reduction
     // TODO:  investigate if testing x < 0.0f is more efficient
-    if (bit_cast<float, unsigned>(x) & 0x80000000u) // test sign bit of x
+    if (bitcast<unsigned int, float>(x) & 0x80000000u) // test sign bit of x
         r = float(M_PI) - r;
     return copysignf(r, y);
 #else
@@ -1905,7 +1943,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE float fast_atan2 (float y, float x) {
 template<typename T>
 OIIO_FORCEINLINE OIIO_HOSTDEVICE T fast_log2 (const T& xval) {
     using namespace simd;
-    typedef typename T::int_t intN;
+    typedef typename T::vint_t intN;
     // See float fast_log2 for explanations
     T x = clamp (xval, T(std::numeric_limits<float>::min()), T(std::numeric_limits<float>::max()));
     intN bits = bitcast_to_int(x);
@@ -1929,9 +1967,9 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE float fast_log2 (const float& xval) {
     // NOTE: clamp to avoid special cases and make result "safe" from large negative values/nans
     float x = clamp (xval, std::numeric_limits<float>::min(), std::numeric_limits<float>::max());
     // based on https://github.com/LiraNuna/glsl-sse2/blob/master/source/vec4.h
-    unsigned bits = bit_cast<float, unsigned>(x);
+    unsigned int bits = bitcast<unsigned int, float>(x);
     int exponent = int(bits >> 23) - 127;
-    float f = bit_cast<unsigned, float>((bits & 0x007FFFFF) | 0x3f800000) - 1.0f;
+    float f = bitcast<float, unsigned int>((bits & 0x007FFFFF) | 0x3f800000) - 1.0f;
     // Examined 2130706432 values of log2 on [1.17549435e-38,3.40282347e+38]: 0.0797524457 avg ulp diff, 3713596 max ulp, 7.62939e-06 max error
     // ulp histogram:
     //  0  = 97.46%
@@ -1988,7 +2026,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE float fast_logb (float x) {
     x = fabsf(x);
     if (x < std::numeric_limits<float>::min()) x = std::numeric_limits<float>::min();
     if (x > std::numeric_limits<float>::max()) x = std::numeric_limits<float>::max();
-    unsigned bits = bit_cast<float, unsigned>(x);
+    unsigned int bits = bitcast<unsigned int, float>(x);
     return float (int(bits >> 23) - 127);
 #else
     return logbf(x);
@@ -2013,7 +2051,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE float fast_log1p (float x) {
 template<typename T>
 OIIO_FORCEINLINE OIIO_HOSTDEVICE T fast_exp2 (const T& xval) {
     using namespace simd;
-    typedef typename T::int_t intN;
+    typedef typename T::vint_t intN;
 #if OIIO_SIMD_SSE
     // See float specialization for explanations
     T x = clamp (xval, T(-126.0f), T(126.0f));
@@ -2045,7 +2083,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE T fast_exp2 (const T& xval) {
 
 template<>
 OIIO_FORCEINLINE OIIO_HOSTDEVICE float fast_exp2 (const float& xval) {
-#if OIIO_NON_INTEL_CLANG && OIIO_FMATH_SIMD_FRIENDLY
+#if OIIO_ANY_CLANG && !OIIO_INTEL_LLVM_COMPILER && OIIO_FMATH_SIMD_FRIENDLY
     // Clang was unhappy using the bitcast/memcpy/reinter_cast/union inside
     // an explicit SIMD loop, so revert to calling the standard version.
     return std::exp2(xval);
@@ -2068,7 +2106,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE float fast_exp2 (const float& xval) {
     r = madd(x, r, 1.0f);
     // multiply by 2 ^ m by adding in the exponent
     // NOTE: left-shift of negative number is undefined behavior
-    return bit_cast<unsigned, float>(bit_cast<float, unsigned>(r) + (unsigned(m) << 23));
+    return bitcast<float, unsigned int>(bitcast<unsigned int, float>(r) + (unsigned(m) << 23));
     // Clang: loop not vectorized: unsafe dependent memory operations in loop.
     // This is why we special case the OIIO_FMATH_SIMD_FRIENDLY above.
     // FIXME: as clang releases continue to improve, periodically check if
@@ -2191,7 +2229,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE float fast_safe_pow (float x, float y) {
     if (OIIO_UNLIKELY(x < 0.0f)) {
         // if x is negative, only deal with integer powers
         // powf returns NaN for non-integers, we will return 0 instead
-        int ybits = bit_cast<float, int>(y) & 0x7fffffff;
+        int ybits = bitcast<int, float>(y) & 0x7fffffff;
         if (ybits >= 0x4b800000) {
             // always even int, keep positive
         } else if (ybits >= 0x3f800000) {
@@ -2199,7 +2237,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE float fast_safe_pow (float x, float y) {
             int k = (ybits >> 23) - 127;  // get exponent
             int j =  ybits >> (23 - k);   // shift out possible fractional bits
             if ((j << (23 - k)) == ybits) // rebuild number and check for a match
-                sign = bit_cast<int, float>(0x3f800000 | (j << 31)); // +1 for even, -1 for odd
+                sign = bitcast<float, int>(0x3f800000 | (j << 31)); // +1 for even, -1 for odd
             else
                 return 0.0f; // not integer
         } else {
@@ -2222,7 +2260,7 @@ OIIO_FORCEINLINE OIIO_HOSTDEVICE float fast_cbrt (float x) {
 #ifndef __CUDA_ARCH__
     float x0 = fabsf(x);
     // from hacker's delight
-    float a = bit_cast<int, float>(0x2a5137a0 + bit_cast<float, int>(x0) / 3); // Initial guess.
+    float a = bitcast<float, int>(0x2a5137a0 + bitcast<int, float>(x0) / 3); // Initial guess.
     // Examined 14272478 values of cbrt on [-9.99999935e-39,9.99999935e-39]: 8.14687e-14 max error
     // Examined 2131958802 values of cbrt on [9.99999935e-39,3.40282347e+38]: 2.46930719 avg ulp diff, 12 max ulp
     a = 0.333333333f * (2.0f * a + x0 / (a * a));  // Newton step.
