@@ -18,6 +18,7 @@
 #include <map>
 #include <sstream>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include <OpenImageIO/export.h>
@@ -26,29 +27,10 @@
 #include <OpenImageIO/platform.h>
 #include <OpenImageIO/string_view.h>
 
+#include <OpenImageIO/detail/fmt.h>
 #include <OpenImageIO/detail/farmhash.h>
 
-#if OIIO_GNUC_VERSION >= 70000
-#    pragma GCC diagnostic push
-#    pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
-#endif
-#ifndef FMT_HEADER_ONLY
-#    define FMT_HEADER_ONLY
-#endif
-#ifndef FMT_EXCEPTIONS
-#    define FMT_EXCEPTIONS 0
-#endif
-#ifndef FMT_USE_GRISU
-#    define FMT_USE_GRISU 1
-#endif
-#include "detail/fmt/ostream.h"
-#include "detail/fmt/format.h"
-#include "detail/fmt/printf.h"
-#if OIIO_GNUC_VERSION >= 70000
-#    pragma GCC diagnostic pop
-#endif
-
-// Allow client software to know if this version of OIIO as Strutil::sprintf
+// Allow client software to know if this version of OIIO has Strutil::sprintf
 #define OIIO_HAS_SPRINTF 1
 
 // Allow client software to know (and to determine, by setting this value
@@ -68,6 +50,11 @@
 #    define OIIO_FORMAT_DEPRECATED
 #endif
 
+// If OIIO_PRINT_IS_SYNCHRONIZED is not defined, assume unsynchronized.
+#ifndef OIIO_PRINT_IS_SYNCHRONIZED
+#    define OIIO_PRINT_IS_SYNCHRONIZED 0
+#endif
+
 // Allow client software to know that at this moment, the fmt-based string
 // formatting is locale-independent. This was 0 in older versions when fmt
 // was locale dependent.
@@ -81,13 +68,16 @@ OIIO_NAMESPACE_BEGIN
 /// @brief     String-related utilities.
 namespace Strutil {
 
-/// Output the string to the file/stream in a synchronized fashion, so that
-/// buffers are flushed and internal mutex is used to prevent threads from
-/// clobbering each other -- output strings coming from concurrent threads
-/// may be interleaved, but each string is "atomic" and will never splice
-/// each other character-by-character.
-void OIIO_UTIL_API sync_output (FILE *file, string_view str);
-void OIIO_UTIL_API sync_output (std::ostream &file, string_view str);
+/// Output the string to the file/stream in a synchronized fashion, with an
+/// internal mutex used to prevent threads from clobbering each other --
+/// output strings coming from concurrent threads may be interleaved, but each
+/// string is "atomic" and will never splice each other
+/// character-by-character. If `flush` is true, the underlying stream will be
+/// flushed after the string is output.
+void OIIO_UTIL_API sync_output (FILE* file, string_view str,
+                                bool flush = true);
+void OIIO_UTIL_API sync_output (std::ostream& file, string_view str,
+                                bool flush = true);
 
 
 /// Construct a std::string in a printf-like fashion.  For example:
@@ -100,10 +90,7 @@ void OIIO_UTIL_API sync_output (std::ostream &file, string_view str);
 /// conventions (in particular, '.' as decimal separator for float values).
 #ifdef OIIO_DOXYGEN
 template<typename Str, typename... Args>
-inline std::string sprintf(const Str& fmt, Args&&... args)
-{
-    return ::fmt::sprintf(fmt, args...);
-}
+std::string sprintf(const Str& fmt, Args&&... args);
 #else
 using ::fmt::sprintf;
 #endif
@@ -137,6 +124,7 @@ using ::fmt::sprintf;
 
 namespace fmt {
 template<typename Str, typename... Args>
+OIIO_NODISCARD
 inline std::string format(const Str& fmt, Args&&... args)
 {
 #if FMT_VERSION >= 70000
@@ -161,7 +149,7 @@ template<typename... Args>
 OIIO_FORMAT_DEPRECATED
 inline std::string format (string_view fmt, const Args&... args)
 {
-    return format (fmt.c_str(), args...);
+    return format ({ fmt.data(), fmt.size() }, args...);
 }
 } // namespace old
 
@@ -206,35 +194,88 @@ inline void fprintf (std::ostream &file, const char* fmt, const Args&... args)
 
 
 
-/// Strutil::print (fmt, ...)
-/// Strutil::print (FILE*, fmt, ...)
-/// Strutil::print (ostream& fmt, ...)
-///
-/// Output formatted strings to stdout, a FILE*, or a stream, respectively.
-/// All use "Python-like" formatting description (as {fmt} does, and some
-/// day, std::format), are type-safe, are thread-safe (the outputs are
-/// "atomic", at least versus other calls to Strutil::*printf), and
-/// automatically flush their outputs. They are all locale-independent by
-/// default (use {:n} for locale-aware formatting).
+namespace sync {
 
-template<typename... Args>
-inline void print (const char* fmt, const Args&... args)
-{
-    sync_output (stdout, Strutil::fmt::format(fmt, args...));
-}
+/// Strutil::sync::print (fmt, ...)
+/// Strutil::sync::print (FILE*, fmt, ...)
+/// Strutil::sync::print (ostream& fmt, ...)
+///
+/// Output formatted strings to stdout, a FILE*, or a stream, using a
+/// "Python-like/std::format" type-safe formatting description. Results are
+/// locale-independent (use `{:n}` locale-aware formatting).
+///
+/// Output is fully thread-safe (the outputs are "atomic" to the file or
+/// stream), and if the stream is buffered, it is flushed after the output).
 
 template<typename... Args>
 inline void print (FILE *file, const char* fmt, const Args&... args)
 {
-    sync_output (file, Strutil::fmt::format(fmt, args...));
+    sync_output (file, ::fmt::format(fmt, args...));
+}
+
+template<typename... Args>
+inline void print (const char* fmt, const Args&... args)
+{
+    print(stdout, fmt, args...);
 }
 
 template<typename... Args>
 inline void print (std::ostream &file, const char* fmt, const Args&... args)
 {
-    sync_output (file, Strutil::fmt::format(fmt, args...));
+    sync_output (file, ::fmt::format(fmt, args...));
+}
+} // namespace sync
+
+
+
+/// Strutil::print (fmt, ...)
+/// Strutil::print (FILE*, fmt, ...)
+/// Strutil::print (ostream& fmt, ...)
+///
+/// Output formatted strings to stdout, a FILE*, or a stream, using a
+/// "Python-like/std::format" type-safe formatting description. Results are
+/// locale-independent (use `{:n}` locale-aware formatting).
+///
+/// As wrappers around fmt::print (https://fmt.dev), these appear currently to
+/// be thread-safe and "atomic" (multiple concurrent calls using the same
+/// stream should not interleave their characters within the output strings).
+/// But we can't 100% guarantee that on all platforms and all versions of fmt.
+/// See the `Strutil::sync::print()` for similar functionality that is
+/// guaranteed to be thread-safe and atomic, as well as flush their outputs
+/// fully after each call (but are, as expected, slower). If
+/// `OIIO_PRINT_IS_SYNCHRONIZED` is defined to be 1 prior to including
+/// `<OpenImageIO/strutil.h>`, then print will be also be thread-safe and
+/// atomic, like `sync::print`.
+#ifdef OIIO_DOXYGEN
+template<typename... Args>
+void print (const char* fmt, const Args&... args);
+
+template<typename... Args>
+void print (FILE *file, const char* fmt, const Args&... args);
+
+template<typename... Args>
+void print (std::ostream &file, const char* fmt, const Args&... args);
+
+#elif FMT_VERSION >= 70000 && !OIIO_PRINT_IS_SYNCHRONIZED
+using ::fmt::print;
+#else
+using sync::print;
+#endif
+
+
+namespace pvt {
+OIIO_UTIL_API void debug(string_view str);
 }
 
+/// `debug(format, ...)` prints debugging message when attribute "debug" is
+/// nonzero, which it is by default for DEBUG compiles or when the environment
+/// variable OPENIMAGEIO_DEBUG is set. This is preferred to raw output to
+/// stderr for debugging statements.
+template<typename... Args>
+void debug(const char* fmt, Args&&... args)
+{
+    pvt::debug(fmt::format(fmt, std::forward<Args>(args)...));
+}
 
 
 
@@ -318,7 +359,7 @@ strhash (size_t len, const char *s)
 /// Hash a string_view. This is OIIO's default favorite string hasher.
 /// Currently, it uses farmhash, is constexpr (for C++14), and works in
 /// Cuda. This is rigged, though, so that empty strings hash always hash to
-/// 0 (that isn't would a raw farmhash would give you, but it's a useful
+/// 0 (that isn't what a raw farmhash would give you, but it's a useful
 /// property, especially for trivial initialization).
 inline OIIO_CONSTEXPR14 size_t
 strhash (string_view s)
@@ -684,7 +725,7 @@ int extract_from_list_string (std::vector<T, Allocator> &vals,
         if (nvals == 0)
             vals.push_back (v);
         else if (valuestrings[i].size()) {
-            if (vals.size() > i)  // don't replace non-existant entries
+            if (vals.size() > i)  // don't replace non-existent entries
                 vals[i] = from_string<T> (valuestrings[i]);
         }
         /* Otherwise, empty space between commas, so leave default alone */
@@ -792,55 +833,46 @@ struct OIIO_UTIL_API StringILess {
 
 
 
-#ifdef _WIN32
-/// Conversion functions between UTF-8 and UTF-16 for windows.
-///
-/// For historical reasons, the standard encoding for strings on windows is
-/// UTF-16, whereas the unix world seems to have settled on UTF-8.  These two
-/// encodings can be stored in std::string and std::wstring respectively, with
-/// the caveat that they're both variable-width encodings, so not all the
-/// standard string methods will make sense (for example std::string::size()
-/// won't return the number of glyphs in a UTF-8 string, unless it happens to
-/// be made up of only the 7-bit ASCII subset).
-///
-/// The standard windows API functions usually have two versions, a UTF-16
-/// version with a 'W' suffix (using wchar_t* strings), and an ANSI version
-/// with a 'A' suffix (using char* strings) which uses the current windows
-/// code page to define the encoding.  (To make matters more confusing there is
-/// also a further "TCHAR" version which is #defined to the UTF-16 or ANSI
-/// version, depending on whether UNICODE is defined during compilation.
-/// This is meant to make it possible to support compiling libraries in
-/// either unicode or ansi mode from the same codebase.)
-///
-/// Using std::string as the string container (as in OIIO) implies that we
-/// can't use UTF-16.  It also means we need a variable-width encoding to
-/// represent characters in non-Latin alphabets in an unambiguous way; the
-/// obvious candidate is UTF-8.  File paths in OIIO are considered to be
-/// represented in UTF-8, and must be converted to UTF-16 before passing to
-/// windows API file opening functions.
-
-/// On the other hand, the encoding used for the ANSI versions of the windows
-/// API is the current windows code page.  This is more compatible with the
-/// default setup of the standard windows command prompt, and may be more
-/// appropriate for error messages.
-
-// Conversion to wide char
-//
+/// Conversion of normal char-based strings (presumed to be UTF-8 encoding)
+/// to wide char string, wstring.
 std::wstring OIIO_UTIL_API utf8_to_utf16 (string_view utf8str) noexcept;
 
-// Conversion from wide char
-//
+/// Conversion from wstring UTF-16 to a UTF-8 std::string.  This is the
+/// standard way to convert from Windows wide character strings used for
+/// filenames into the UTF-8 strings OIIO expects for filenames when passed to
+/// functions like ImageInput::open().
 std::string OIIO_UTIL_API utf16_to_utf8(const std::wstring& utf16str) noexcept;
-#endif
+
 
 
 /// Copy at most size characters (including terminating 0 character) from
 /// src into dst[], filling any remaining characters with 0 values. Returns
 /// dst. Note that this behavior is identical to strncpy, except that it
-/// guarantees that there will be a termining 0 character.
+/// guarantees that there will be a terminating 0 character.
 OIIO_UTIL_API char * safe_strcpy (char *dst, string_view src, size_t size) noexcept;
 
 
+/// Append `src` to the end of the C-string buffer dst, plus a terminating
+/// null, assuming that dst can hold at most `size` characters (including
+/// terminating 0 character). Returns dst. Note that this behavior is similar
+/// to strcat, but guarantees that the resulting string fits into `size` bytes
+/// and is null-terminated.
+OIIO_UTIL_API char* safe_strcat(char *dst, string_view src, size_t size) noexcept;
+
+
+/// Return the length of null-terminated string `str`, up to maximum `size`.
+/// If str is nullptr, return 0. This is equivalent to C11 strnlen_s.
+OIIO_UTIL_API size_t safe_strlen(const char* str, size_t size) noexcept;
+
+
+/// Is the character a whitespace character (space, linefeed, tab, carrage
+/// return)? Note: this is safer than C isspace(), which has undefined
+/// behavior for negative char values. Also note that it differs from C
+/// isspace by not detecting form feed or vertical tab, because who cares.
+inline bool isspace(char c) {
+    return c == ' ' || c == '\n' || c == '\t' || c == '\r';
+}
+
 /// Modify str to trim any leading whitespace (space, tab, linefeed, cr)
 /// from the front.
 void OIIO_UTIL_API skip_whitespace (string_view &str) noexcept;
