@@ -29,26 +29,21 @@ OIIO_PLUGIN_NAMESPACE_BEGIN
 class JpgOutput final : public ImageOutput {
 public:
     JpgOutput() { init(); }
-    virtual ~JpgOutput() { close(); }
-    virtual const char* format_name(void) const override { return "jpeg"; }
-    virtual int supports(string_view feature) const override
+    ~JpgOutput() override { close(); }
+    const char* format_name(void) const override { return "jpeg"; }
+    int supports(string_view feature) const override
     {
         return (feature == "exif" || feature == "iptc" || feature == "ioproxy");
     }
-    virtual bool open(const std::string& name, const ImageSpec& spec,
-                      OpenMode mode = Create) override;
-    virtual bool write_scanline(int y, int z, TypeDesc format, const void* data,
-                                stride_t xstride) override;
-    virtual bool write_tile(int x, int y, int z, TypeDesc format,
-                            const void* data, stride_t xstride,
-                            stride_t ystride, stride_t zstride) override;
-    virtual bool close() override;
-    virtual bool copy_image(ImageInput* in) override;
-    virtual bool set_ioproxy(Filesystem::IOProxy* ioproxy) override
-    {
-        m_io = ioproxy;
-        return true;
-    }
+    bool open(const std::string& name, const ImageSpec& spec,
+              OpenMode mode = Create) override;
+    bool write_scanline(int y, int z, TypeDesc format, const void* data,
+                        stride_t xstride) override;
+    bool write_tile(int x, int y, int z, TypeDesc format, const void* data,
+                    stride_t xstride, stride_t ystride,
+                    stride_t zstride) override;
+    bool close() override;
+    bool copy_image(ImageInput* in) override;
 
 private:
     std::string m_filename;
@@ -60,8 +55,6 @@ private:
     jvirt_barray_ptr* m_copy_coeffs;
     struct jpeg_decompress_struct* m_copy_decompressor;
     std::vector<unsigned char> m_tilebuffer;
-    std::unique_ptr<Filesystem::IOProxy> m_io_local;
-    Filesystem::IOProxy* m_io = nullptr;
     // m_outbuffer/m_outsize are used for jpeg-to-memory
     unsigned char* m_outbuffer = nullptr;
 #if OIIO_JPEG_LIB_VERSION >= 94
@@ -78,8 +71,7 @@ private:
     {
         m_copy_coeffs       = NULL;
         m_copy_decompressor = NULL;
-        m_io_local.reset();
-        m_io = nullptr;
+        ioproxy_clear();
         clear_outbuffer();
     }
 
@@ -151,22 +143,13 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
         return false;
     }
 
-    if (auto ioparam = m_spec.find_attribute("oiio:ioproxy", TypeDesc::PTR))
-        m_io = ioparam->get<Filesystem::IOProxy*>();
-    if (!m_io) {
-        // If no proxy was supplied, create a file reader
-        m_io = new Filesystem::IOFile(name, Filesystem::IOProxy::Write);
-        m_io_local.reset(m_io);
-    }
-    if (!m_io || m_io->mode() != Filesystem::IOProxy::Write) {
-        errorfmt("Could not open \"{}\"", name);
-        m_io = nullptr;
-        m_io_local.reset();
+    ioproxy_retrieve_from_config(m_spec);
+    if (!ioproxy_use_or_open(name))
         return false;
-    }
 
     m_cinfo.err = jpeg_std_error(&c_jerr);  // set error handler
     jpeg_create_compress(&m_cinfo);         // create compressor
+    Filesystem::IOProxy* m_io = ioproxy();
     if (!strcmp(m_io->proxytype(), "file")) {
         auto fd = reinterpret_cast<Filesystem::IOFile*>(m_io)->handle();
         jpeg_stdio_dest(&m_cinfo, fd);  // set output stream
@@ -180,12 +163,25 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
     m_cinfo.image_height = m_spec.height;
 
     // JFIF can only handle grayscale and RGB. Do the best we can with this
-    // limited format by truncating to 3 channels if > 3 are requested,
-    // truncating to 1 channel if 2 are requested.
+    // limited format by switching to 1 or 3 channels.
     if (m_spec.nchannels >= 3) {
+        // For 3 or more channels, write the first 3 as RGB and drop any
+        // additional channels.
         m_cinfo.input_components = 3;
         m_cinfo.in_color_space   = JCS_RGB;
+    } else if (m_spec.nchannels == 2) {
+        // Two channels are tricky. If the first channel name is "Y", assume
+        // it's a luminance image and write it as a single-channel grayscale.
+        // Otherwise, punt, write it as an RGB image with third channel black.
+        if (m_spec.channel_name(0) == "Y") {
+            m_cinfo.input_components = 1;
+            m_cinfo.in_color_space   = JCS_GRAYSCALE;
+        } else {
+            m_cinfo.input_components = 3;
+            m_cinfo.in_color_space   = JCS_RGB;
+        }
     } else {
+        // One channel, assume it's grayscale
         m_cinfo.input_components = 1;
         m_cinfo.in_color_space   = JCS_GRAYSCALE;
     }
@@ -238,11 +234,10 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
     m_next_scanline = 0;  // next scanline we'll write
 
     // Write JPEG comment, if sent an 'ImageDescription'
-    ParamValue* comment = m_spec.find_attribute("ImageDescription",
-                                                TypeDesc::STRING);
-    if (comment && comment->data()) {
-        const char** c = (const char**)comment->data();
-        jpeg_write_marker(&m_cinfo, JPEG_COM, (JOCTET*)*c, strlen(*c) + 1);
+    std::string comment = m_spec.get_string_attribute("ImageDescription");
+    if (comment.size()) {
+        jpeg_write_marker(&m_cinfo, JPEG_COM, (JOCTET*)comment.c_str(),
+                          comment.size() + 1);
     }
 
     if (Strutil::iequals(m_spec.get_string_attribute("oiio:ColorSpace"), "sRGB"))
@@ -259,7 +254,8 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
     exif.push_back(0);
     exif.push_back(0);
     encode_exif(m_spec, exif);
-    jpeg_write_marker(&m_cinfo, JPEG_APP0 + 1, (JOCTET*)&exif[0], exif.size());
+    jpeg_write_marker(&m_cinfo, JPEG_APP0 + 1, (JOCTET*)exif.data(),
+                      exif.size());
 
     // Write IPTC IIM metadata tags, if we have anything
     std::vector<char> iptc;
@@ -278,14 +274,14 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
         head.push_back((char)(iptc.size() >> 8));  // size of block
         head.push_back((char)(iptc.size() & 0xff));
         iptc.insert(iptc.begin(), head.begin(), head.end());
-        jpeg_write_marker(&m_cinfo, JPEG_APP0 + 13, (JOCTET*)&iptc[0],
+        jpeg_write_marker(&m_cinfo, JPEG_APP0 + 13, (JOCTET*)iptc.data(),
                           iptc.size());
     }
 
     // Write XMP packet, if we have anything
     std::string xmp = encode_xmp(m_spec, true);
     if (!xmp.empty()) {
-        static char prefix[] = "http://ns.adobe.com/xap/1.0/";
+        static char prefix[] = "http://ns.adobe.com/xap/1.0/";  //NOSONAR
         std::vector<char> block(prefix, prefix + strlen(prefix) + 1);
         block.insert(block.end(), xmp.c_str(), xmp.c_str() + xmp.length());
         jpeg_write_marker(&m_cinfo, JPEG_APP0 + 1, (JOCTET*)&block[0],
@@ -307,7 +303,7 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
             if ((unsigned int)(num_markers * MAX_DATA_BYTES_IN_MARKER)
                 != icc_profile_length)
                 num_markers++;
-            int curr_marker     = 1; /* per spec, count strarts at 1*/
+            int curr_marker     = 1; /* per spec, count starts at 1*/
             size_t profile_size = MAX_DATA_BYTES_IN_MARKER + ICC_HEADER_SIZE;
             std::vector<unsigned char> profile(profile_size);
             while (icc_profile_length > 0) {
@@ -317,12 +313,13 @@ JpgOutput::open(const std::string& name, const ImageSpec& newspec,
                                (unsigned int)MAX_DATA_BYTES_IN_MARKER);
                 icc_profile_length -= length;
                 // Write the JPEG marker header (APP2 code and marker length)
-                strncpy((char*)&profile[0], "ICC_PROFILE", profile_size);
+                strcpy((char*)profile.data(), "ICC_PROFILE");  // NOSONAR
                 profile[11] = 0;
                 profile[12] = curr_marker;
                 profile[13] = (unsigned char)num_markers;
-                memcpy(&profile[0] + ICC_HEADER_SIZE,
-                       icc_profile + length * (curr_marker - 1), length);
+                memcpy(profile.data() + ICC_HEADER_SIZE,
+                       icc_profile + length * (curr_marker - 1),
+                       length);  //NOSONAR
                 jpeg_write_marker(&m_cinfo, JPEG_APP0 + 2, &profile[0],
                                   ICC_HEADER_SIZE + length);
                 curr_marker++;
@@ -471,7 +468,18 @@ JpgOutput::write_scanline(int y, int z, TypeDesc format, const void* data,
     int save_nchannels = m_spec.nchannels;
     m_spec.nchannels   = m_cinfo.input_components;
 
-    data = to_native_scanline(format, data, xstride, m_scratch, m_dither, y, z);
+    if (save_nchannels == 2 && m_spec.nchannels == 3) {
+        // Edge case: expanding 2 channels to 3
+        uint8_t* tmp = OIIO_ALLOCA(uint8_t, m_spec.width * 3);
+        memset(tmp, 0, m_spec.width * 3);
+        convert_image(2, m_spec.width, 1, 1, data, format, xstride, AutoStride,
+                      AutoStride, tmp, TypeDesc::UINT8, 3 * sizeof(uint8_t),
+                      AutoStride, AutoStride);
+        data = tmp;
+    } else {
+        data = to_native_scanline(format, data, xstride, m_scratch, m_dither, y,
+                                  z);
+    }
     m_spec.nchannels = save_nchannels;
 
     jpeg_write_scanlines(&m_cinfo, (JSAMPLE**)&data, 1);
@@ -496,7 +504,7 @@ JpgOutput::write_tile(int x, int y, int z, TypeDesc format, const void* data,
 bool
 JpgOutput::close()
 {
-    if (!m_io) {  // Already closed
+    if (!ioproxy_opened()) {  // Already closed
         init();
         return true;
     }
@@ -539,12 +547,10 @@ JpgOutput::close()
         // have fully general IO overloads, but it can write to memory
         // buffers, we did that, so now we have to copy that in one big chunk
         // to IOProxy.
-        m_io->write(m_outbuffer, m_outsize);
+        ioproxy()->write(m_outbuffer, m_outsize);
     }
 
-    m_io_local.reset();  // closes the file if we opened it locally
     init();
-
     return ok;
 }
 
