@@ -667,7 +667,7 @@ add_exif_item_to_spec(ImageSpec& spec, const char* name,
         for (int i = 0; i < n; ++i) {
             unsigned int num, den;
             num = ((const unsigned int*)dataptr)[2 * i + 0];
-            den = ((const unsigned int*)dataptr)[2 * i + 1];
+            den = ((const unsigned int*)dataptr)[2 * i + 1];  //NOSONAR
             if (swab) {
                 swap_endian(&num);
                 swap_endian(&den);
@@ -686,7 +686,7 @@ add_exif_item_to_spec(ImageSpec& spec, const char* name,
         for (int i = 0; i < n; ++i) {
             int num, den;
             num = ((const int*)dataptr)[2 * i + 0];
-            den = ((const int*)dataptr)[2 * i + 1];
+            den = ((const int*)dataptr)[2 * i + 1];  //NOSONAR
             if (swab) {
                 swap_endian(&num);
                 swap_endian(&den);
@@ -878,7 +878,7 @@ read_exif_tag(ImageSpec& spec, const TIFFDirEntry* dirp, cspan<uint8_t> buf,
     } else {
         // Everything else -- use our table to handle the general case
         const TagInfo* taginfo = tagmap.find(dir.tdir_tag);
-        if (taginfo) {
+        if (taginfo && !spec.extra_attribs.contains(taginfo->name)) {
             if (taginfo->handler)
                 taginfo->handler(*taginfo, dir, buf, spec, swab,
                                  offset_adjustment);
@@ -939,14 +939,17 @@ encode_exif_entry(const ParamValue& p, int tag, std::vector<TIFFDirEntry>& dirs,
     TIFFDataType type = tagmap.tifftype(tag);
     size_t count      = (size_t)tagmap.tiffcount(tag);
     TypeDesc element  = p.type().elementtype();
+    OIIO_DASSERT(p.data() != nullptr);
 
     switch (type) {
     case TIFF_ASCII:
         if (p.type() == TypeDesc::STRING) {
-            const char* s = *(const char**)p.data();
-            int len       = strlen(s) + 1;
-            append_tiff_dir_entry(dirs, data, tag, type, len, s,
-                                  offset_correction, 0, endianreq);
+            ustring s = *(const ustring*)p.data();
+            if (s.size()) {
+                int len = size_t(s.size()) + 1;
+                append_tiff_dir_entry(dirs, data, tag, type, len, s.data(),
+                                      offset_correction, 0, endianreq);
+            }
             return;
         }
         break;
