@@ -5,6 +5,8 @@
 #include <cstdio>
 #include <cstdlib>
 
+#include <OpenImageIO/half.h>
+
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/fmath.h>
 #include <OpenImageIO/hash.h>
@@ -33,7 +35,7 @@ threads_default()
 
 // Global private data
 namespace pvt {
-recursive_mutex imageio_mutex;
+mutex imageio_mutex;
 atomic_int oiio_threads(threads_default());
 atomic_int oiio_exr_threads(threads_default());
 atomic_int oiio_read_chunk(256);
@@ -50,14 +52,7 @@ std::string input_format_list;   // comma-separated list of readable formats
 std::string output_format_list;  // comma-separated list of writable formats
 std::string extension_list;      // list of all extensions for all formats
 std::string library_list;        // list of all libraries for all formats
-static const char* oiio_debug_env = getenv("OPENIMAGEIO_DEBUG");
-#ifdef NDEBUG
-int oiio_print_debug(oiio_debug_env ? Strutil::stoi(oiio_debug_env) : 0);
-#else
-int oiio_print_debug(oiio_debug_env ? Strutil::stoi(oiio_debug_env) : 1);
-#endif
-int oiio_log_times = Strutil::from_string<int>(
-    Sysutil::getenv("OPENIMAGEIO_LOG_TIMES"));
+int oiio_log_times = Strutil::stoi(Sysutil::getenv("OPENIMAGEIO_LOG_TIMES"));
 std::vector<float> oiio_missingcolor;
 }  // namespace pvt
 
@@ -67,8 +62,7 @@ using namespace pvt;
 namespace {
 // Hidden global OIIO data.
 static spin_mutex attrib_mutex;
-static const int maxthreads  = 256;  // reasonable maximum for sanity check
-static FILE* oiio_debug_file = NULL;
+static const int maxthreads = 512;  // reasonable maximum for sanity check
 
 class TimingLog {
 public:
@@ -266,18 +260,7 @@ geterror(bool clear)
 void
 debug(string_view message)
 {
-    recursive_lock_guard lock(pvt::imageio_mutex);
-    if (oiio_print_debug) {
-        if (!oiio_debug_file) {
-            const char* filename = getenv("OPENIMAGEIO_DEBUG_FILE");
-            oiio_debug_file = filename && filename[0] ? fopen(filename, "a")
-                                                      : stderr;
-            OIIO_ASSERT(oiio_debug_file);
-            if (!oiio_debug_file)
-                return;
-        }
-        Strutil::print(oiio_debug_file, "OIIO DEBUG: {}", message);
-    }
+    Strutil::pvt::debug(message);
 }
 
 
@@ -342,6 +325,10 @@ attribute(string_view name, TypeDesc type, const void* val)
         limit_imagesize_MB = *(const int*)val;
         return true;
     }
+    if (name == "use_tbb" && type == TypeInt) {
+        oiio_use_tbb = *(const int*)val;
+        return true;
+    }
     if (name == "debug" && type == TypeInt) {
         oiio_print_debug = *(const int*)val;
         return true;
@@ -442,6 +429,10 @@ getattribute(string_view name, TypeDesc type, void* val)
         *(int*)val = tiff_multithread;
         return true;
     }
+    if (name == "use_tbb" && type == TypeInt) {
+        *(int*)val = oiio_use_tbb;
+        return true;
+    }
     if (name == "debug" && type == TypeInt) {
         *(int*)val = oiio_print_debug;
         return true;
@@ -784,14 +775,13 @@ parallel_convert_image(int nchannels, int width, int height, int depth,
                            nchannels, width, height);
 
     int blocksize = std::max(1, height / nthreads);
-    parallel_for_chunked(
-        0, height, blocksize, [=](int /*id*/, int64_t ybegin, int64_t yend) {
-            convert_image(nchannels, width, yend - ybegin, depth,
-                          (const char*)src + src_ystride * ybegin, src_type,
-                          src_xstride, src_ystride, src_zstride,
-                          (char*)dst + dst_ystride * ybegin, dst_type,
-                          dst_xstride, dst_ystride, dst_zstride);
-        });
+    parallel_for_chunked(0, height, blocksize, [=](int64_t ybegin, int64_t yend) {
+        convert_image(nchannels, width, yend - ybegin, depth,
+                      (const char*)src + src_ystride * ybegin, src_type,
+                      src_xstride, src_ystride, src_zstride,
+                      (char*)dst + dst_ystride * ybegin, dst_type, dst_xstride,
+                      dst_ystride, dst_zstride);
+    });
     return true;
 }
 
