@@ -8,13 +8,14 @@
 #include <cstdlib>
 #include <fcntl.h>
 #include <iostream>
+#include <random>
 #include <regex>
 #include <string>
 
 #include <OpenImageIO/dassert.h>
 #include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/platform.h>
-#include <OpenImageIO/refcnt.h>
+// #include <OpenImageIO/refcnt.h>
 #include <OpenImageIO/strutil.h>
 #include <OpenImageIO/ustring.h>
 
@@ -23,14 +24,23 @@
 #    include <direct.h>
 #    include <io.h>
 #    include <shellapi.h>
+#    include <sys/types.h>
+#    include <sys/utime.h>
 #else
+#    include <sys/stat.h>
+#    include <sys/types.h>
 #    include <unistd.h>
 #endif
 
-#include <boost/filesystem.hpp>
+#if defined(USE_STD_FILESYSTEM)
+#    include <filesystem>
+namespace filesystem = std::filesystem;
+using std::error_code;
+#else
+#    include <boost/filesystem.hpp>
 namespace filesystem = boost::filesystem;
-using error_code     = boost::system::error_code;
-// FIXME: use std::filesystem when available
+using boost::system::error_code;
+#endif
 
 
 
@@ -99,6 +109,15 @@ std::string
 Filesystem::extension(string_view filepath, bool include_dot) noexcept
 {
     std::string s;
+#if !defined(USE_STD_FILESYSTEM)
+    if (filepath.find('.') == 0 && filepath.rfind('.') == 0) {
+        // Work around bug in boost::filesystem::path::extension(), where
+        // ".foo" thinks the extension is foo. But we know, and
+        // std::filesystem::path::extension() knows, that a file called ".foo"
+        // has no extension, that's just the base filename.
+        return s;
+    }
+#endif
     try {
         s = pathstr(u8path(filepath).extension());
     } catch (...) {
@@ -190,6 +209,8 @@ Filesystem::searchpath_split(string_view searchpath, bool validonly)
         size_t len = path.size();
         while (len > 1 && (path.back() == '/' || path.back() == '\\'))
             path.erase(--len);
+        if (path.empty())
+            continue;
         // If it's a valid directory, or if validonly is false, add it
         // to the list
         if (!validonly || Filesystem::is_directory(path))
@@ -221,7 +242,7 @@ Filesystem::searchpath_find(const std::string& filename_utf8,
         const filesystem::path d(u8path(d_utf8));
         filesystem::path f = d / filename;
         error_code ec;
-        if (filesystem::is_regular(f, ec)) {
+        if (filesystem::is_regular_file(f, ec)) {
             return pathstr(f);
         }
 
@@ -423,9 +444,38 @@ Filesystem::temp_directory_path()
 std::string
 Filesystem::unique_path(string_view model)
 {
+#ifdef USE_BOOST_FILESYSTEM
+    // Boost filesystem has unique_path().
     error_code ec;
     filesystem::path p = filesystem::unique_path(u8path(model), ec);
     return ec ? std::string() : pathstr(p);
+#else
+    // std::filesystem does not have unique_path(). Punt!
+#    if defined(_WIN32)
+    // boost internally doesn't use MultiByteToWideChar (CP_UTF8,...
+    // to convert char* to wchar_t* because they do not know the encoding
+    // See boost/filesystem/path.hpp
+    // The only correct way to do this is to do the conversion ourselves
+    std::wstring modelStr    = Strutil::utf8_to_utf16(model);
+#    else
+    std::string modelStr = model.str();
+#    endif
+    static const char chrs[] = "0123456789abcdef";
+    static std::mt19937 rg { std::random_device {}() };
+    static std::uniform_int_distribution<size_t> pick(0, 15);
+    static std::mutex mutex;
+    std::lock_guard<std::mutex> lock(mutex);
+    std::string name;
+    while (true) {
+        name = model.str();
+        for (size_t i = 0, e = model.size(); i < e; ++i)
+            if (name[i] == '%')
+                name[i] = chrs[pick(rg)];
+        if (!exists(name))
+            break;
+    }
+    return name;
+#endif
 }
 
 
@@ -451,7 +501,7 @@ Filesystem::fopen(string_view path, string_view mode)
     return ::_wfopen(wpath.c_str(), wmode.c_str());
 #else
     // on Unix platforms passing in UTF-8 works
-    return ::fopen(path.c_str(), mode.c_str());
+    return ::fopen(std::string(path).c_str(), std::string(mode).c_str());
 #endif
 }
 
@@ -492,7 +542,7 @@ Filesystem::open(OIIO::ifstream& stream, string_view path,
     stream.open(wpath.c_str(), mode);
     stream.seekg(0, std::ios_base::beg);  // force seek, otherwise broken
 #else
-    stream.open(path.c_str(), mode);
+    stream.open(path, mode);
 #endif
 }
 
@@ -508,7 +558,7 @@ Filesystem::open(OIIO::ofstream& stream, string_view path,
     std::wstring wpath = Strutil::utf8_to_utf16(path);
     stream.open(wpath.c_str(), mode);
 #else
-    stream.open(path.c_str(), mode);
+    stream.open(path, mode);
 #endif
 }
 
@@ -524,7 +574,7 @@ Filesystem::open(string_view path, int flags)
     return ::_wopen(wpath.c_str(), flags);
 #else
     // on Unix platforms passing in UTF-8 works
-    return ::open(path.c_str(), flags);
+    return ::open(std::string(path).c_str(), flags);
 #endif
 }
 
@@ -584,9 +634,20 @@ Filesystem::read_bytes(string_view path, void* buffer, size_t n, size_t pos)
 std::time_t
 Filesystem::last_write_time(string_view path) noexcept
 {
-    error_code ec;
-    std::time_t t = filesystem::last_write_time(u8path(path), ec);
-    return ec ? 0 : t;
+#ifdef _WIN32
+    struct __stat64 st;
+    auto r = _wstat64(u8path(path).c_str(), &st);
+#else
+    struct stat st;
+    auto r = stat(u8path(path).c_str(), &st);
+#endif
+    if (r == 0) {
+        // success
+        return st.st_mtime;
+    } else {
+        // failure
+        return 0;
+    }
 }
 
 
@@ -594,8 +655,19 @@ Filesystem::last_write_time(string_view path) noexcept
 void
 Filesystem::last_write_time(string_view path, std::time_t time) noexcept
 {
-    error_code ec;
-    filesystem::last_write_time(u8path(path), time, ec);
+#ifdef _WIN32
+    struct _utimbuf times;
+    times.actime  = time;
+    times.modtime = time;
+    _wutime(u8path(path).c_str(), &times);
+#else
+    struct timespec times[2];
+    times[0].tv_sec  = 0;
+    times[0].tv_nsec = UTIME_OMIT;
+    times[1].tv_sec  = time;
+    times[1].tv_nsec = 0;
+    utimensat((int)AT_FDCWD, u8path(path).c_str(), times, AT_SYMLINK_NOFOLLOW);
+#endif
 }
 
 
@@ -919,7 +991,7 @@ Filesystem::scan_for_matching_filenames(const std::string& pattern_,
              !ec && it != end_it; ++it) {
             std::string itpath = Filesystem::generic_filepath(
                 it->path().string());
-            if (filesystem::is_regular(itpath, ec)) {
+            if (filesystem::is_regular_file(itpath, ec)) {
                 const std::string f = pathstr(itpath);
                 std::match_results<std::string::const_iterator> frame_match;
                 if (regex_match(f, frame_match, pattern_re)) {
@@ -1032,8 +1104,7 @@ Filesystem::IOFile::IOFile(string_view filename, Mode mode)
 {
     // Call Filesystem::fopen since it handles UTF-8 file paths on Windows,
     // which std fopen does not.
-    m_file = Filesystem::fopen(m_filename.c_str(),
-                               m_mode == Write ? "wb" : "rb");
+    m_file = Filesystem::fopen(m_filename, m_mode == Write ? "wb" : "rb");
     if (!m_file) {
         m_mode          = Closed;
         int e           = errno;
@@ -1089,6 +1160,15 @@ Filesystem::IOFile::read(void* buf, size_t size)
         return 0;
     size_t r = fread(buf, 1, size, m_file);
     m_pos += r;
+    if (r < size) {
+        if (feof(m_file))
+            error("end of file");
+        else if (ferror(m_file)) {
+            int e           = errno;
+            const char* msg = e ? std::strerror(e) : nullptr;
+            error(msg ? msg : "unknown error");
+        }
+    }
     return r;
 }
 
