@@ -5,7 +5,6 @@
 #include <fstream>
 #include <sstream>
 
-#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/filesystem.h>
 #include <OpenImageIO/imageio.h>
 #include <OpenImageIO/platform.h>
@@ -36,6 +35,13 @@ test_filename_decomposition()
                      ".ext");
     OIIO_CHECK_EQUAL(Filesystem::extension("/directory/filename"), "");
     OIIO_CHECK_EQUAL(Filesystem::extension("/directory/filename."), ".");
+    OIIO_CHECK_EQUAL(Filesystem::extension("a.foo"), ".foo");
+    OIIO_CHECK_EQUAL(Filesystem::extension("a.foo", false), "foo");
+    OIIO_CHECK_EQUAL(Filesystem::extension("foo"), "");
+    OIIO_CHECK_EQUAL(Filesystem::extension("foo", false), "");
+    OIIO_CHECK_EQUAL(Filesystem::extension(".foo"), "");
+    OIIO_CHECK_EQUAL(Filesystem::extension(".foo", false), "");
+
     OIIO_CHECK_EQUAL(Filesystem::parent_path(test), "/directoryA/directory");
 
     std::cout << "Testing path_is_absolute\n";
@@ -80,6 +86,25 @@ test_filename_searchpath_find()
 
     std::cout << "Testing searchpath_split\n";
     std::vector<std::string> dirs;
+
+    // Split of empty string should make an empty path vector
+    dirs.clear();
+    Filesystem::searchpath_split("", dirs, false);
+    OIIO_CHECK_EQUAL(dirs.size(), 0);
+
+    // Test that empty paths don't show up in the result vector
+    dirs.clear();
+    Filesystem::searchpath_split(":", dirs, false);
+    OIIO_CHECK_EQUAL(dirs.size(), 0);
+    Filesystem::searchpath_split("::", dirs, false);
+    OIIO_CHECK_EQUAL(dirs.size(), 0);
+    dirs.clear();
+    Filesystem::searchpath_split(":abc::def:", dirs, false);
+    OIIO_CHECK_EQUAL(dirs.size(), 2);
+    OIIO_CHECK_EQUAL(dirs[0], "abc");
+    OIIO_CHECK_EQUAL(dirs[1], "def");
+
+    dirs.clear();
     Filesystem::searchpath_split(pathlist, dirs);
     OIIO_CHECK_EQUAL(dirs.size(), 3);
     OIIO_CHECK_EQUAL(dirs[0], ".." DIRSEP "..");
@@ -203,7 +228,7 @@ test_seq(const char* str, const char* expected)
 
 
 static void
-test_file_seq(const char* pattern, const char* override,
+test_file_seq(const char* pattern, string_view overrideval,
               const std::string& expected)
 {
     std::vector<int> numbers;
@@ -212,23 +237,21 @@ test_file_seq(const char* pattern, const char* override,
     std::string frame_range;
 
     Filesystem::parse_pattern(pattern, 0, normalized_pattern, frame_range);
-    if (override && strlen(override) > 0)
-        frame_range = override;
+    if (overrideval.size())
+        frame_range = overrideval;
     Filesystem::enumerate_sequence(frame_range, numbers);
     Filesystem::enumerate_file_sequence(normalized_pattern, numbers, names);
     std::string joined = Strutil::join(names, " ");
-    std::cout << "  " << pattern;
-    if (override)
-        std::cout << " + " << override;
-    std::cout << " -> " << joined << "\n";
+    Strutil::print(" {}{}{} -> {}\n", pattern, overrideval.size() ? " + " : "",
+                   overrideval, joined);
     OIIO_CHECK_EQUAL(joined, expected);
 }
 
 
 
 static void
-test_file_seq_with_view(const char* pattern, const char* override,
-                        const char* view, const std::string& expected)
+test_file_seq_with_view(const char* pattern, string_view overrideval,
+                        string_view view, const std::string& expected)
 {
     std::vector<int> numbers;
     std::vector<string_view> views;
@@ -237,11 +260,11 @@ test_file_seq_with_view(const char* pattern, const char* override,
     std::string frame_range;
 
     Filesystem::parse_pattern(pattern, 0, normalized_pattern, frame_range);
-    if (override && strlen(override) > 0)
-        frame_range = override;
-    Filesystem::enumerate_sequence(frame_range.c_str(), numbers);
+    if (overrideval.size())
+        frame_range = overrideval;
+    Filesystem::enumerate_sequence(frame_range, numbers);
 
-    if (view) {
+    if (view.size()) {
         for (size_t i = 0, e = numbers.size(); i < e; ++i)
             views.emplace_back(view);
     }
@@ -249,10 +272,8 @@ test_file_seq_with_view(const char* pattern, const char* override,
     Filesystem::enumerate_file_sequence(normalized_pattern, numbers, views,
                                         names);
     std::string joined = Strutil::join(names, " ");
-    std::cout << "  " << pattern;
-    if (override)
-        std::cout << " + " << override;
-    std::cout << " -> " << joined << "\n";
+    Strutil::print(" {}{}{} -> {}\n", pattern, overrideval.size() ? " + " : "",
+                   overrideval, joined);
     OIIO_CHECK_EQUAL(joined, expected);
 }
 
@@ -523,6 +544,19 @@ test_mem_proxies()
 
 
 
+void
+test_last_write_time()
+{
+    Filesystem::write_text_file("oiio-testtime.txt", "test");
+    time_t t = Filesystem::last_write_time("oiio-testtime.txt");
+    std::cout << "Last write time of oiio-testtime.txt is " << t << "\n";
+    Filesystem::last_write_time("oiio-testtime.txt", t - 42);
+    OIIO_CHECK_EQUAL(Filesystem::last_write_time("oiio-testtime.txt"), t - 42);
+    Filesystem::remove("oiio-testtime.txt");
+}
+
+
+
 int
 main(int /*argc*/, char* /*argv*/[])
 {
@@ -532,6 +566,7 @@ main(int /*argc*/, char* /*argv*/[])
     test_frame_sequences();
     test_scan_sequences();
     test_mem_proxies();
+    test_last_write_time();
 
     return unit_test_failures;
 }
