@@ -28,6 +28,7 @@
 #include <OpenImageIO/platform.h>
 #include <OpenImageIO/string_view.h>
 #include <OpenImageIO/strutil.h>
+#include <OpenImageIO/thread.h>
 #include <OpenImageIO/ustring.h>
 
 #ifdef _WIN32
@@ -91,15 +92,14 @@ isdigit(char c)
 
 
 
-OIIO_NO_SANITIZE_ADDRESS
-const char*
-string_view::c_str() const
+OIIO_NO_SANITIZE_ADDRESS const char*
+c_str(string_view str)
 {
     // Usual case: either empty, or null-terminated
-    if (m_len == 0)  // empty string
+    if (str.size() == 0)  // empty string
         return "";
 
-    // This clause attempts to find out if there's a string-teriminating
+    // This clause attempts to find out if there's a string-terminating
     // '\0' character just beyond the boundary of the string_view, in which
     // case, simply returning m_chars (with no ustring creation) is a valid
     // C string.
@@ -117,38 +117,74 @@ string_view::c_str() const
     // in C++17 string_view. So maybe we'll find ourselves relying on it a
     // lot less, and therefore the performance hit of doing it foolproof
     // won't be as onerous.
-    if (m_chars[m_len] == 0)  // 0-terminated
-        return m_chars;
+    if (str[str.size()] == 0)  // 0-terminated
+        return str.data();
 
     // Rare case: may not be 0-terminated. Bite the bullet and construct a
     // 0-terminated string.  We use ustring as a way to avoid any issues of
     // who cleans up the allocation, though it means that it will stay in
     // the ustring table forever. Punt on this for now, it's an edge case
     // that we need to handle, but is not likely to ever be an issue.
-    return ustring(m_chars, 0, m_len).c_str();
+    return ustring(str).c_str();
 }
 
 
 
 void
-Strutil::sync_output(FILE* file, string_view str)
+Strutil::sync_output(FILE* file, string_view str, bool flush)
 {
     if (str.size() && file) {
         std::unique_lock<std::mutex> lock(output_mutex);
         fwrite(str.data(), 1, str.size(), file);
-        fflush(file);
+        if (flush)
+            fflush(file);
     }
 }
 
 
 
 void
-Strutil::sync_output(std::ostream& file, string_view str)
+Strutil::sync_output(std::ostream& file, string_view str, bool flush)
 {
     if (str.size()) {
         std::unique_lock<std::mutex> lock(output_mutex);
         file << str;
-        file.flush();
+        if (flush)
+            file.flush();
+    }
+}
+
+
+
+namespace pvt {
+static const char* oiio_debug_env = getenv("OPENIMAGEIO_DEBUG");
+#ifdef NDEBUG
+OIIO_UTIL_API int
+    oiio_print_debug(oiio_debug_env ? Strutil::stoi(oiio_debug_env) : 0);
+#else
+OIIO_UTIL_API int
+    oiio_print_debug(oiio_debug_env ? Strutil::stoi(oiio_debug_env) : 1);
+#endif
+}  // namespace pvt
+
+
+void
+Strutil::pvt::debug(string_view message)
+{
+    if (OIIO::pvt::oiio_print_debug) {
+        static mutex debug_mutex;
+        lock_guard lock(debug_mutex);
+        static FILE* oiio_debug_file = nullptr;
+        if (!oiio_debug_file) {
+            const char* filename = getenv("OPENIMAGEIO_DEBUG_FILE");
+            oiio_debug_file = filename && filename[0] ? fopen(filename, "a")
+                                                      : stderr;
+            OIIO_ASSERT(oiio_debug_file);
+            if (!oiio_debug_file)
+                oiio_debug_file = stderr;
+        }
+        print(oiio_debug_file, "OIIO DEBUG: {}", message);
+        fflush(oiio_debug_file);
     }
 }
 
@@ -312,7 +348,7 @@ Strutil::escape_chars(string_view unescaped)
             case '\f': c = 'f'; break;
             case '\a': c = 'a'; break;
             }
-            s.insert(i, &c, 1);
+            s.insert(i, 1, c);
         }
     }
     return s;
@@ -369,7 +405,7 @@ Strutil::wordwrap(string_view src, int columns, int prefix, string_view sep,
         return src;  // give up, no way to make it wrap
     std::ostringstream out;
     columns -= prefix;  // now columns is the real width we have to work with
-    std::string allsep = Strutil::concat(sep, presep);
+    std::string allsep = Strutil::sprintf("%s%s", sep, presep);
     while ((int)src.length() > columns) {
         // Find the last `sep` break before the column limit.
         size_t breakpoint = src.find_last_of(allsep, columns);
@@ -1729,7 +1765,7 @@ Strutil::scan_datetime(string_view str, int& year, int& month, int& day,
                   || parse_char(str, '/', false))
               && parse_int(str, day) && parse_int(str, hour)
               && parse_char(str, ':', false) && parse_int(str, min)
-              && parse_char(str, ':', false) && parse_int(str, sec);
+              && parse_char(str, ':', false) && parse_int(str, sec);  //NOSONAR
     return ok && month >= 1 && month <= 12 && day >= 1 && day <= 31 && hour >= 0
            && hour <= 23 && min >= 0 && min <= 59 && sec >= 0 && sec <= 59;
 }
