@@ -295,7 +295,7 @@ test_get_rest_arguments()
 void
 test_escape(string_view raw, string_view escaped)
 {
-    Strutil::print("escape '{}' <-> '{}'\n", raw, escaped);
+    Strutil::printf ("escape '%s' <-> '%s'\n", raw, escaped);
     OIIO_CHECK_EQUAL(Strutil::escape_chars(raw), escaped);
     OIIO_CHECK_EQUAL(Strutil::unescape_chars(escaped), raw);
 }
@@ -482,7 +482,7 @@ test_comparisons()
     std::string abcnope = "1abcdefghijklmnopqrstuvwxyz";
     std::string haystack = std::string("begin") + abc + "oiio"
                          + Strutil::repeat(abc, 10) + "123" + abc + "end";
-    bench ("string== success", [&](){ DoNotOptimize(abc == abc); });
+    bench ("string== success", [&](){ DoNotOptimize(abc == abc); }); // NOSONAR
     bench ("string== failure", [&](){ DoNotOptimize(abc == abcmore); });
     bench ("iequals success", [&](){ DoNotOptimize(Strutil::iequals(abc, abc)); });
     bench ("iless easy", [&](){ DoNotOptimize(Strutil::iless(abc, abcnope)); });
@@ -1062,6 +1062,56 @@ test_safe_strcpy()
 
 
 
+void
+test_safe_strcat()
+{
+    std::cout << "Testing safe_strcat\n";
+    const size_t len = 8;
+    {  // test in-bounds copy
+        char result[len + 1] = { 100, 101, 102, 103, 104, 105, 106, 107, 108 };
+        Strutil::safe_strcpy(result, "123", len);
+        Strutil::safe_strcat(result, "ABC", len);
+        OIIO_CHECK_EQUAL(std::string(result), "123ABC");
+        OIIO_CHECK_EQUAL(result[6], 0);
+        OIIO_CHECK_EQUAL(result[7], 0);
+        OIIO_CHECK_EQUAL(result[8], 108);
+    }
+    {  // test over-bounds copy
+        char result[len + 1] = { 100, 101, 102, 103, 104, 105, 106, 107, 108 };
+        Strutil::safe_strcpy(result, "123", len);
+        Strutil::safe_strcat(result, "ABCDEF", len);
+        OIIO_CHECK_EQUAL(std::string(result), "123ABCD");
+        OIIO_CHECK_EQUAL(result[7], 0);
+        OIIO_CHECK_EQUAL(result[8], 108);
+    }
+    {  // test empty string copy
+        char result[len + 1] = { 100, 101, 102, 103, 104, 105, 106, 107, 108 };
+        Strutil::safe_strcpy(result, "123", len);
+        Strutil::safe_strcat(result, "", len);
+        OIIO_CHECK_EQUAL(std::string(result), "123");
+        OIIO_CHECK_EQUAL(result[3], 0);
+        OIIO_CHECK_EQUAL(result[4], 0);
+        OIIO_CHECK_EQUAL(result[5], 0);
+        OIIO_CHECK_EQUAL(result[6], 0);
+        OIIO_CHECK_EQUAL(result[7], 0);
+        OIIO_CHECK_EQUAL(result[8], 108);
+    }
+    {  // test NULL case
+        char result[len + 1] = { 100, 101, 102, 103, 104, 105, 106, 107, 108 };
+        Strutil::safe_strcpy(result, "123", len);
+        Strutil::safe_strcat(result, nullptr, len);
+        OIIO_CHECK_EQUAL(std::string(result), "123");
+        OIIO_CHECK_EQUAL(result[3], 0);
+        OIIO_CHECK_EQUAL(result[4], 0);
+        OIIO_CHECK_EQUAL(result[5], 0);
+        OIIO_CHECK_EQUAL(result[6], 0);
+        OIIO_CHECK_EQUAL(result[7], 0);
+        OIIO_CHECK_EQUAL(result[8], 108);
+    }
+}
+
+
+
 // test some of the trickier methods in string_view.
 void
 test_string_view()
@@ -1076,6 +1126,24 @@ test_string_view()
     OIIO_CHECK_EQUAL(string_view(s), cstr);  // test ctr from std::string
     OIIO_CHECK_EQUAL(sr, cstr);  // These better be the same
 
+#ifdef OIIO_STD_STRING_VIEW_AVAILABLE
+    {
+        std::cout << "  Testing OIIO::string_view <-> std::string_view\n";
+        std::string_view ssv = sr;
+        OIIO::string_view osv = ssv;
+        OIIO_CHECK_EQUAL(osv, sr);
+    }
+#endif
+
+#ifdef OIIO_EXPERIMENTAL_STRING_VIEW_AVAILABLE
+    {
+        std::cout << "  Testing OIIO::string_view <-> std::experimental::string_view\n";
+        std::experimental::string_view ssv = sr;
+        OIIO::string_view osv = ssv;
+        OIIO_CHECK_EQUAL(osv, sr);
+    }
+#endif
+
     OIIO_CHECK_EQUAL(sr.substr(0), sr); // whole string
     OIIO_CHECK_EQUAL(sr.substr(2), "23401234"); // nonzero pos, default n
     OIIO_CHECK_EQUAL(sr.substr(2, 3), "234"); // true substrng
@@ -1146,6 +1214,11 @@ test_string_view()
     OIIO_CHECK_EQUAL(sr.find_last_not_of("234", 5), 1);
     OIIO_CHECK_EQUAL(sr.find_last_not_of("xyz"), 9);
     OIIO_CHECK_EQUAL(sr.find_last_not_of("01234"), string_view::npos);
+
+    // Just force template expansion of wchar_t variety to make sure it's
+    // not horribly broken.
+    wstring_view wsv;
+    OIIO_CHECK_ASSERT(wsv == wsv);
 }
 
 
@@ -1360,7 +1433,7 @@ test_float_formatting()
         sstream.precision(9);
         sstream << *f;
         char buffer[64];
-        std::sprintf(buffer, "%.9g", *f);
+        std::snprintf(buffer, sizeof(buffer), "%.9g", *f);
         std::string tiny = Strutil::sprintf("%.9g", *f);
         if (sstream.str() != tiny || tiny != buffer)
             Strutil::printf(
@@ -1491,6 +1564,7 @@ main(int /*argc*/, char* /*argv*/[])
     test_to_string();
     test_extract();
     test_safe_strcpy();
+    test_safe_strcat();
     test_string_view();
     test_parse();
     test_locale();
