@@ -93,6 +93,30 @@ template<unsigned BASE_CAPACITY, unsigned POOL_SIZE> struct TableRepMap {
         }
     }
 
+    // Look up based on hash only. Return nullptr if not found. Note that if
+    // the hash is not unique, this will return the first entry that matches
+    // the hash.
+    const char* lookup(size_t hash)
+    {
+        ustring_read_lock_t lock(mutex);
+#ifdef USTRING_TRACK_NUM_LOOKUPS
+        // NOTE: this simple increment adds a substantial amount of overhead
+        // so keep it off by default, unless the user really wants it
+        // NOTE2: note that in debug, asserts like the one in ustring::from_unique
+        // can skew the number of lookups compared to release builds
+        ++num_lookups;
+#endif
+        size_t pos = hash & mask, dist = 0;
+        for (;;) {
+            if (entries[pos] == 0)
+                return 0;
+            if (entries[pos]->hashed == hash)
+                return entries[pos]->c_str();
+            ++dist;
+            pos = (pos + dist) & mask;  // quadratic probing
+        }
+    }
+
     const char* insert(string_view str, size_t hash)
     {
         ustring_write_lock_t lock(mutex);
@@ -157,7 +181,7 @@ private:
 
     char* pool_alloc(size_t len)
     {
-        // round up to nearest multiple of pointer size to guarentee proper alignment of TableRep objects
+        // round up to nearest multiple of pointer size to guarantee proper alignment of TableRep objects
         len = (len + alignof(ustring::TableRep) - 1)
               & ~(alignof(ustring::TableRep) - 1);
 
@@ -200,6 +224,8 @@ struct UstringTable {
         return whichbin(hash).lookup(str, hash);
     }
 
+    const char* lookup(size_t hash) { return whichbin(hash).lookup(hash); }
+
     const char* insert(string_view str, size_t hash)
     {
         return whichbin(hash).insert(str, hash);
@@ -233,7 +259,7 @@ struct UstringTable {
 
 private:
     enum {
-        // NOTE: this guarentees NUM_BINS is a power of 2
+        // NOTE: this guarantees NUM_BINS is a power of 2
         BIN_SHIFT = 12,
         NUM_BINS  = 1 << BIN_SHIFT,
         TOP_SHIFT = 8 * sizeof(size_t) - BIN_SHIFT
@@ -362,7 +388,7 @@ ustring::TableRep::TableRep(string_view strref, size_t hash)
     // It turns out that the first field of a gcc std::string is a pointer
     // to the characters within the basic_string::_Rep.  We merely redirect
     // that pointer, though for std::string to function properly, the chars
-    // must be preceeded immediately in memory by the rest of
+    // must be preceded immediately in memory by the rest of
     // basic_string::_Rep, consisting of length, capacity and refcount
     // fields.  And we have designed our TableRep to do just that!  So now
     // we redirect the std::string's pointer to our own characters and its
@@ -391,7 +417,7 @@ ustring::TableRep::TableRep(string_view strref, size_t hash)
     // allocations to be had for short strings, so we just let those behave
     // as normal.  But if it's going to make a long string (we can tell from
     // the length), we construct it ourselves, forcing the pointer to point
-    // to the charcters in the TableRep we allocated.
+    // to the characters in the TableRep we allocated.
     if (length >= libcpp_string__min_cap /* it'll be a "long string" */) {
         ((libcpp_string__long*)&str)->__cap_ = libcpp_string__long_mask
                                                | (length + 1);
@@ -427,7 +453,7 @@ const char*
 ustring::make_unique(string_view strref)
 {
     UstringTable& table(ustring_table());
-    // Eliminate NULLs
+    // Eliminate nullptr-referred string views
     if (!strref.data())
         strref = string_view("", 0);
 
@@ -442,6 +468,18 @@ ustring::make_unique(string_view strref)
     const char* result = table.lookup(strref, hash);
     if (result)
         return result;
+    auto nul = strref.find('\0');
+    if (nul != string_view::npos) {
+        // Strutil::print("ustring::make_unique: string contains nulls @{}/{}: \"{}\"\n",
+        //                strref.find('\0'), strref.size(), strref);
+        // OIIO_ASSERT(strref.find('\0') == string_view::npos &&
+        //             "ustring::make_unique() does not support embedded nulls");
+        strref = strref.substr(0, nul);
+        hash   = Strutil::strhash(strref);
+        result = table.lookup(strref, hash);
+        if (result)
+            return result;
+    }
     // Strutil::print("ADDED ustring \"{}\" {:08x}\n", strref, hash);
     return table.insert(strref, hash);
 
@@ -454,6 +492,21 @@ ustring::make_unique(string_view strref)
     if (result)
         return result;
 
+    // ustring doesn't allow strings with embedded nul characters. Before we
+    // go any further, trim beyond any nul and rehash.
+    auto nul = strref.find('\0');
+    if (nul != string_view::npos) {
+        // Strutil::print("ustring::make_unique: string contains nulls @{}/{}: \"{}\"\n",
+        //                strref.find('\0'), strref.size(), strref);
+        // OIIO_ASSERT(strref.find('\0') == string_view::npos &&
+        //             "ustring::make_unique() does not support embedded nulls");
+        strref = strref.substr(0, nul);
+        hash   = Strutil::strhash(strref);
+        result = table.lookup(strref, hash);
+        if (result)
+            return result;
+    }
+
     // We did not find it. There are two possibilities: (1) the string is in
     // the table but has a different hash because it collided; or (2) the
     // string is not yet in the table.
@@ -490,7 +543,7 @@ ustring::make_unique(string_view strref)
             result = rev->second;
             break;
         }
-        // Rehash, but keep the bin bits identical so we alwasy rehash into
+        // Rehash, but keep the bin bits identical so we always rehash into
         // the same (locked) bin.
         hash = (hash & binmask)
                | (farmhash::Fingerprint(hash) & rm.nobin_mask());
@@ -515,6 +568,15 @@ ustring::make_unique(string_view strref)
 
 
 
+ustring
+ustring::from_hash(size_t hash)
+{
+    UstringTable& table(ustring_table());
+    return from_unique(table.lookup(hash));
+}
+
+
+
 ustring
 ustring::concat(string_view s, string_view t)
 {
