@@ -7,10 +7,11 @@
 
 #include <functional>
 #include <memory>
+#include <stack>
 
 #include <boost/container/flat_set.hpp>
 
-#include <OpenImageIO/Imath.h>
+#include <OpenImageIO/half.h>
 
 #include <OpenImageIO/argparse.h>
 #include <OpenImageIO/color.h>
@@ -106,6 +107,24 @@ public:
     ImageSpec input_config;  // configuration options for reading
     ImageSpec first_input_dimensions;
     std::string input_channel_set;  // Optional input channel set
+    ParamValueList uservars;        // User-defined variables (with --set)
+    ArgParse ap;                    // Command-line argument parser
+
+    struct ControlRec {
+        std::string command;  // control command: "if", "while", etc.
+        int start_arg;        // argument number of the loop/condition start
+        bool condition;       // was the condition true?
+        bool running;         // are we currently running in this block?
+
+        ControlRec(string_view command, int start, bool condition, bool running)
+            : command(command)
+            , start_arg(start)
+            , condition(condition)
+            , running(running)
+        {
+        }
+    };
+    std::stack<ControlRec> control_stack;
 
     // Output options
     TypeDesc output_dataformat;  // Requested output data format
@@ -162,6 +181,30 @@ public:
     void clear_options();
     void clear_input_config();
 
+    // Process command line arguments
+    void getargs(int argc, char* argv[]);
+
+    bool running() const
+    {
+        return control_stack.empty()
+               || (control_stack.top().running
+                   && control_stack.top().condition);
+    }
+
+    void push_control(string_view command, int start, bool cond)
+    {
+        control_stack.emplace(command, start, cond, cond & running());
+        ap.running(running());
+    }
+
+    ControlRec pop_control()
+    {
+        ControlRec ctrl = control_stack.top();
+        control_stack.pop();
+        ap.running(running());
+        return ctrl;
+    }
+
     /// Force img to be read at this point.  Use this wrapper, don't directly
     /// call img->read(), because there's extra work done here specific to
     /// oiiotool.
@@ -275,20 +318,6 @@ public:
     void error(string_view command, string_view message = "") const;
     void warning(string_view command, string_view message = "") const;
 
-    // Formatted errors with printf-like notation
-    template<typename... Args>
-    void errorf(string_view command, const char* fmt, const Args&... args) const
-    {
-        error(command, Strutil::sprintf(fmt, args...));
-    }
-
-    template<typename... Args>
-    void warningf(string_view command, const char* fmt,
-                  const Args&... args) const
-    {
-        warning(command, Strutil::sprintf(fmt, args...));
-    }
-
     // Formatted errors with std::format-like notation
     template<typename... Args>
     void errorfmt(string_view command, const char* fmt,
@@ -550,11 +579,20 @@ public:
     /// Error reporting for ImageRec: call this with printf-like arguments.
     /// Note however that this is fully typesafe!
     template<typename... Args>
+    OIIO_DEPRECATED("Use errorfmt instead")
     void errorf(const char* fmt, const Args&... args) const
     {
         append_error(Strutil::sprintf(fmt, args...));
     }
 
+    /// Error reporting for ImageRec: call this with printf-like arguments.
+    /// Note however that this is fully typesafe!
+    template<typename... Args>
+    void errorfmt(const char* fmt, const Args&... args) const
+    {
+        append_error(Strutil::fmt::format(fmt, args...));
+    }
+
     /// Return true if the IR has had an error and has an error message
     /// to retrieve via geterror().
     bool has_error(void) const;
@@ -623,16 +661,6 @@ print_stats(std::ostream& out, Oiiotool& ot, const std::string& filename,
             ROI roi = {});
 
 
-// Set an attribute of the given image.  The type should be one of
-// TypeDesc::INT (decode the value as an int), FLOAT, STRING, or UNKNOWN
-// (look at the string and try to discern whether it's an int, float, or
-// string).  If the 'value' string is empty, it will delete the
-// attribute.  If allsubimages is true, apply the attribute to all
-// subimages, otherwise just the first subimage.
-bool
-set_attribute(ImageRecRef img, string_view attribname, TypeDesc type,
-              string_view value, bool allsubimages);
-
 inline bool
 same_size(const ImageBuf& A, const ImageBuf& B)
 {
@@ -783,6 +811,13 @@ public:
         : OiiotoolOp(ot, opname, argc, argv, ninputs, {}, impl_func)
     {
     }
+    OiiotoolOp(Oiiotool& ot, string_view opname, cspan<const char*> argv,
+               int ninputs, setup_func_t setup_func = nullptr,
+               impl_func_t impl_func = nullptr)
+        : OiiotoolOp(ot, opname, (int)argv.size(), (const char**)argv.data(),
+                     ninputs, setup_func, impl_func)
+    {
+    }
     virtual ~OiiotoolOp() {}
 
     // The operator(), function-call mode, does most of the work. Although
@@ -820,7 +855,14 @@ public:
             // Read the inputs
             subimages = compute_subimages();
             // Initialize the output image
-            m_ir[0] = new_output_imagerec();
+            if (inplace() && nimages() >= 2) {
+                // If instructed to operate in place, just make the output
+                // another reference to the first input image.
+                m_ir[0] = m_ir[1];
+            } else {
+                // Not in-place, so make a new output image.
+                m_ir[0] = new_output_imagerec();
+            }
             ot.push(m_ir[0]);
         }
 
@@ -846,10 +888,10 @@ public:
         cleanup();
 
         if (ot.debug) {
-            Strutil::print("    {} took {}  (total time {}, mem {})\n",
-                           opname(), Strutil::timeintervalformat(timer(), 2),
-                           Strutil::timeintervalformat(ot.total_runtime(), 2),
-                           Strutil::memformat(Sysutil::memory_used()));
+            Strutil::printf("    %s took %s  (total time %s, mem %s)\n",
+                            opname(), Strutil::timeintervalformat(timer(), 2),
+                            Strutil::timeintervalformat(ot.total_runtime(), 2),
+                            Strutil::memformat(Sysutil::memory_used()));
         }
         return 0;
     }
@@ -870,7 +912,7 @@ public:
                     // Call the impl kernel for this subimage
                     bool ok = impl(m_img);
                     if (!ok)
-                        ot.errorf(opname(), "%s", m_img[0]->geterror());
+                        ot.errorfmt(opname(), "{}", m_img[0]->geterror());
 
                     // Merge metadata if called for
                     if (ot.metamerge)
@@ -888,7 +930,7 @@ public:
             // Make sure to forward any errors missed by the impl
             for (auto& im : m_img)
                 if (im->has_error())
-                    ot.errorf(opname(), "%s", im->geterror());
+                    ot.errorfmt(opname(), "{}", im->geterror());
         }
     }
 
@@ -1018,6 +1060,7 @@ public:
     string_view args(int i) const { return m_args[i]; }
     int nimages() const { return m_nimages; }
     string_view opname() const { return m_opname; }
+    ParamValueList& options() { return m_options; }
     const ParamValueList& options() const { return m_options; }
     ImageBuf* img(int i) const { return m_img[i]; }
 
@@ -1047,6 +1090,11 @@ public:
     void skip_impl(bool val) { m_skip_impl = val; }
     bool skip_impl() const { return m_skip_impl; }
 
+    // Call inplace(true) if the operation should work on the single top
+    // image *in place*, i.e. not copying to form a new image.
+    void inplace(bool val) { m_inplace = val; }
+    bool inplace() const { return m_inplace; }
+
 protected:
     Oiiotool& ot;
     std::string m_opname;
@@ -1054,6 +1102,7 @@ protected:
     int m_nimages;
     bool m_preserve_miplevels = false;
     bool m_skip_impl          = false;
+    bool m_inplace            = false;
     std::vector<ImageRecRef> m_ir;
     std::vector<ImageBuf*> m_img;
     std::vector<string_view> m_args;
