@@ -29,6 +29,7 @@
 #    undef copysign
 #endif
 
+#include <OpenImageIO/Imath.h>
 #include <OpenImageIO/deepdata.h>
 #include <OpenImageIO/imagebuf.h>
 #include <OpenImageIO/imagecache.h>
@@ -132,7 +133,10 @@ template<> struct PyTypeForCType<int> { typedef py::int_ type; };
 template<> struct PyTypeForCType<unsigned int> { typedef py::int_ type; };
 template<> struct PyTypeForCType<short> { typedef py::int_ type; };
 template<> struct PyTypeForCType<unsigned short> { typedef py::int_ type; };
+template<> struct PyTypeForCType<char> { typedef py::int_ type; };
+template<> struct PyTypeForCType<unsigned char> { typedef py::int_ type; };
 template<> struct PyTypeForCType<int64_t> { typedef py::int_ type; };
+template<> struct PyTypeForCType<uint64_t> { typedef py::int_ type; };
 template<> struct PyTypeForCType<float> { typedef py::float_ type; };
 template<> struct PyTypeForCType<half> { typedef py::float_ type; };
 template<> struct PyTypeForCType<double> { typedef py::float_ type; };
@@ -323,6 +327,12 @@ py_buffer_to_stdvector(std::vector<T>& vals, const py::buffer& obj)
         } else if (std::is_same<T, unsigned int>::value
                    && binfo.format.basetype == TypeDesc::UINT) {
             vals.emplace_back(T(binfo.dataval<unsigned int>(i)));
+        } else if (std::is_same<T, unsigned char>::value
+                   && binfo.format.basetype == TypeDesc::UINT8) {
+            vals.emplace_back(T(binfo.dataval<unsigned char>(i)));
+        } else if (std::is_same<T, unsigned short>::value
+                   && binfo.format.basetype == TypeDesc::UINT16) {
+            vals.emplace_back(T(binfo.dataval<unsigned short>(i)));
         } else {
             // FIXME? Other cases?
             vals.emplace_back(T(42));
@@ -368,7 +378,7 @@ py_to_stdvector(std::vector<T>& vals, const py::object& obj)
         return py_indexable_pod_to_stdvector(vals, obj.cast<py::list>());
     }
     // Apparently a str can masquerade as a buffer object, so make sure to
-    // exclude that from teh buffer case.
+    // exclude that from the buffer case.
     if (py::isinstance<py::buffer>(obj) && !py::isinstance<py::str>(obj)) {
         return py_buffer_to_stdvector(vals, obj.cast<py::buffer>());
     }
@@ -402,6 +412,18 @@ C_to_tuple(const T* vals, size_t size)
 }
 
 
+template<>
+inline py::tuple
+C_to_tuple(cspan<unsigned char> vals)
+{
+    size_t size = vals.size();
+    py::tuple result(size);
+    for (size_t i = 0; i < size; ++i)
+        result[i] = static_cast<unsigned char>(vals[i]);
+    return result;
+}
+
+
 // Special case for TypeDesc
 template<>
 inline py::tuple
@@ -423,6 +445,7 @@ template<typename T>
 inline py::object
 C_to_val_or_tuple(const T* vals, TypeDesc type, int nvalues = 1)
 {
+    OIIO_DASSERT(vals && nvalues);
     size_t n = type.numelements() * type.aggregate * nvalues;
     if (n == 1 && !type.arraylen)
         return typename PyTypeForCType<T>::type(vals[0]);
@@ -452,6 +475,14 @@ attribute_typed(T& myobj, string_view name, TypeDesc type, const POBJ& dataobj)
             myobj.attribute(name, type, &vals[0]);
         return ok;
     }
+    if (type.basetype == TypeDesc::UINT8) {
+        std::vector<unsigned char> vals;
+        bool ok = py_to_stdvector(vals, dataobj);
+        ok &= (vals.size() == type.numelements() * type.aggregate);
+        if (ok)
+            myobj.attribute(name, type, &vals[0]);
+        return ok;
+    }
     if (type.basetype == TypeDesc::FLOAT) {
         std::vector<float> vals;
         bool ok = py_to_stdvector(vals, dataobj);
@@ -496,6 +527,8 @@ getattribute_typed(const T& obj, const std::string& name,
         return C_to_val_or_tuple((const short*)data, type);
     if (type.basetype == TypeDesc::UINT16)
         return C_to_val_or_tuple((const unsigned short*)data, type);
+    if (type.basetype == TypeDesc::UINT8)
+        return C_to_val_or_tuple((const unsigned char*)data, type);
     if (type.basetype == TypeDesc::FLOAT)
         return C_to_val_or_tuple((const float*)data, type);
     if (type.basetype == TypeDesc::DOUBLE)
@@ -504,6 +537,10 @@ getattribute_typed(const T& obj, const std::string& name,
         return C_to_val_or_tuple((const half*)data, type);
     if (type.basetype == TypeDesc::STRING)
         return C_to_val_or_tuple((const char**)data, type);
+    if (type.basetype == TypeDesc::INT64)
+        return C_to_val_or_tuple((const int64_t*)data, type);
+    if (type.basetype == TypeDesc::UINT64)
+        return C_to_val_or_tuple((const uint64_t*)data, type);
     return py::none();
 }
 
@@ -596,7 +633,7 @@ case TypeDesc::TYPE:                                                       \
                              nvals)
 
     switch (t.basetype) {
-        // ParamValue_convert_dispatch(UCHAR);
+        ParamValue_convert_dispatch(UCHAR);
         // ParamValue_convert_dispatch(CHAR);
         ParamValue_convert_dispatch(USHORT);
         ParamValue_convert_dispatch(SHORT);
@@ -627,6 +664,8 @@ delegate_setitem(C& self, const std::string& key, py::object obj)
         self[key] = int(obj.template cast<py::int_>());
     else if (py::isinstance<py::str>(obj))
         self[key] = std::string(obj.template cast<py::str>());
+    else if (py::isinstance<py::bytes>(obj))
+        self[key] = std::string(obj.template cast<py::bytes>());
     else
         throw std::invalid_argument("Bad type for __setitem__");
 }
