@@ -72,72 +72,65 @@ struct TIFF_tag_info {
 class TIFFInput final : public ImageInput {
 public:
     TIFFInput();
-    virtual ~TIFFInput();
-    virtual const char* format_name(void) const override { return "tiff"; }
-    virtual bool valid_file(const std::string& filename) const override;
-    virtual int supports(string_view feature) const override
+    ~TIFFInput() override;
+    const char* format_name(void) const override { return "tiff"; }
+    bool valid_file(const std::string& filename) const override;
+    int supports(string_view feature) const override
     {
         return (feature == "exif" || feature == "iptc" || feature == "ioproxy");
         // N.B. No support for arbitrary metadata.
     }
-    virtual bool open(const std::string& name, ImageSpec& newspec) override;
-    virtual bool open(const std::string& name, ImageSpec& newspec,
-                      const ImageSpec& config) override;
-    virtual bool close() override;
-    virtual int current_subimage(void) const override
+    bool open(const std::string& name, ImageSpec& newspec) override;
+    bool open(const std::string& name, ImageSpec& newspec,
+              const ImageSpec& config) override;
+    bool close() override;
+    int current_subimage(void) const override
     {
         // If m_emulate_mipmap is true, pretend subimages are mipmap levels
         lock_guard lock(*this);
         return m_emulate_mipmap ? 0 : m_subimage;
     }
-    virtual int current_miplevel(void) const override
+    int current_miplevel(void) const override
     {
         // If m_emulate_mipmap is true, pretend subimages are mipmap levels
         lock_guard lock(*this);
         return m_emulate_mipmap ? m_subimage : 0;
     }
-    virtual bool seek_subimage(int subimage, int miplevel) override;
-    virtual ImageSpec spec(int subimage, int miplevel) override;
-    virtual ImageSpec spec_dimensions(int subimage, int miplevel) override;
+    bool seek_subimage(int subimage, int miplevel) override;
+    ImageSpec spec(int subimage, int miplevel) override;
+    ImageSpec spec_dimensions(int subimage, int miplevel) override;
     const ImageSpec& spec(void) const override { return m_spec; }
-    virtual bool read_native_scanline(int subimage, int miplevel, int y, int z,
-                                      void* data) override;
-    virtual bool read_native_scanlines(int subimage, int miplevel, int ybegin,
-                                       int yend, int z, void* data) override;
-    virtual bool read_native_tile(int subimage, int miplevel, int x, int y,
-                                  int z, void* data) override;
-    virtual bool read_native_tiles(int subimage, int miplevel, int xbegin,
-                                   int xend, int ybegin, int yend, int zbegin,
-                                   int zend, void* data) override;
-    virtual bool read_scanline(int y, int z, TypeDesc format, void* data,
-                               stride_t xstride) override;
-    virtual bool read_scanlines(int subimage, int miplevel, int ybegin,
-                                int yend, int z, int chbegin, int chend,
-                                TypeDesc format, void* data, stride_t xstride,
-                                stride_t ystride) override;
-    virtual bool read_tile(int x, int y, int z, TypeDesc format, void* data,
-                           stride_t xstride, stride_t ystride,
-                           stride_t zstride) override;
-    virtual bool read_tiles(int subimage, int miplevel, int xbegin, int xend,
-                            int ybegin, int yend, int zbegin, int zend,
-                            int chbegin, int chend, TypeDesc format, void* data,
-                            stride_t xstride, stride_t ystride,
-                            stride_t zstride) override;
-    virtual bool set_ioproxy(Filesystem::IOProxy* ioproxy) override
-    {
-        m_io = ioproxy;
-        return true;
-    }
+    bool read_native_scanline(int subimage, int miplevel, int y, int z,
+                              void* data) override;
+    bool read_native_scanlines(int subimage, int miplevel, int ybegin, int yend,
+                               int z, void* data) override;
+    bool read_native_tile(int subimage, int miplevel, int x, int y, int z,
+                          void* data) override;
+    bool read_native_tiles(int subimage, int miplevel, int xbegin, int xend,
+                           int ybegin, int yend, int zbegin, int zend,
+                           void* data) override;
+    bool read_scanline(int y, int z, TypeDesc format, void* data,
+                       stride_t xstride) override;
+    bool read_scanlines(int subimage, int miplevel, int ybegin, int yend, int z,
+                        int chbegin, int chend, TypeDesc format, void* data,
+                        stride_t xstride, stride_t ystride) override;
+    bool read_tile(int x, int y, int z, TypeDesc format, void* data,
+                   stride_t xstride, stride_t ystride,
+                   stride_t zstride) override;
+    bool read_tiles(int subimage, int miplevel, int xbegin, int xend,
+                    int ybegin, int yend, int zbegin, int zend, int chbegin,
+                    int chend, TypeDesc format, void* data, stride_t xstride,
+                    stride_t ystride, stride_t zstride) override;
 
 private:
     TIFF* m_tif;                            ///< libtiff handle
     std::string m_filename;                 ///< Stash the filename
     std::vector<unsigned char> m_scratch;   ///< Scratch space for us to use
     std::vector<unsigned char> m_scratch2;  ///< More scratch
-    int m_subimage;                  ///< What subimage are we looking at?
-    int m_next_scanline;             ///< Next scanline we'll read
-    bool m_no_random_access;         ///< Should we avoid random access?
-    bool m_emulate_mipmap;           ///< Should we emulate mip with subimage?
+    int m_subimage;           ///< What subimage are we looking at?
+    int m_next_scanline;      ///< Next scanline we'll read, relative to ymin
+    bool m_no_random_access;  ///< Should we avoid random access?
+    bool m_emulate_mipmap;    ///< Should we emulate mip with subimage?
     bool m_keep_unassociated_alpha;  ///< If the image is unassociated, please
                                      ///<   try to keep it that way!
     bool m_raw_color;                ///< If the image is not RGB, don't
@@ -157,7 +150,6 @@ private:
     std::vector<unsigned short> m_colormap;   ///< Color map for palette images
     std::vector<uint32_t> m_rgbadata;         ///< Sometimes we punt
     std::vector<ImageSpec> m_subimage_specs;  ///< Cached subimage specs
-    Filesystem::IOProxy* m_io = nullptr;
 
     // Reset everything to initial state
     void init()
@@ -174,7 +166,7 @@ private:
         m_colormap.clear();
         m_use_rgba_interface = false;
         m_subimage_specs.clear();
-        m_io = nullptr;
+        ioproxy_clear();
     }
 
     // Just close the TIFF file handle, but don't forget anything we
@@ -228,7 +220,7 @@ private:
 #endif
 
     OIIO_NODISCARD
-    TypeDesc tiffgetfieldtype(string_view name OIIO_MAYBE_UNUSED, int tag)
+    TypeDesc tiffgetfieldtype(int tag)
     {
         auto field = find_field(tag);
         if (!field)
@@ -245,7 +237,7 @@ private:
     bool safe_tiffgetfield(string_view name OIIO_MAYBE_UNUSED, int tag,
                            TypeDesc expected, void* dest)
     {
-        TypeDesc type = tiffgetfieldtype(name, tag);
+        TypeDesc type = tiffgetfieldtype(tag);
         // Caller expects a specific type and the tag doesn't match? Punt.
         if (expected != TypeUnknown && !equivalent(expected, type))
             return false;
@@ -295,7 +287,7 @@ private:
         } else if (tiffdatatype == TIFF_ASCII) {
             ok = TIFFGetField(m_tif, tag, &s);
             if (ok && s && *s)
-                count = strlen(s);
+                count = Strutil::safe_strlen(s, 64 * 1024);
         } else {
             // Some other type, we should not have been asking for this
             // as ASCII, or maybe the tag is just the wrong data type in
@@ -315,9 +307,10 @@ private:
     {
         string_view s;
         if (tiff_get_string_field(tag, name, s)) {
-            m_spec.attribute(name, s);
-            // TODO: If the length is 0, erase the attrib rather than
-            // setting it to the empty string.
+            if (s.size())
+                m_spec.attribute(name, s);
+            else
+                m_spec.erase_attribute(name);
         }
     }
 
@@ -412,12 +405,12 @@ private:
     {
         OIIO_DASSERT (m_compression == COMPRESSION_ADOBE_DEFLATE /*||
                       m_compression == COMPRESSION_NONE*/);
+        size_t nvals = size_t(width) * size_t(height) * size_t(channels);
         if (m_compression == COMPRESSION_NONE) {
             // just copy if there's no compression
             memcpy(uncompressed_buf, compressed_buf, csize);
             if (m_is_byte_swapped && m_spec.format == TypeUInt16)
-                TIFFSwabArrayOfShort((unsigned short*)uncompressed_buf,
-                                     width * height * channels);
+                TIFFSwabArrayOfShort((unsigned short*)uncompressed_buf, nvals);
             return;
         }
         uLong uncompressed_size = (uLong)strip_bytes;
@@ -428,8 +421,7 @@ private:
             return;
         }
         if (m_is_byte_swapped && m_spec.format == TypeUInt16)
-            TIFFSwabArrayOfShort((unsigned short*)uncompressed_buf,
-                                 width * height * channels);
+            TIFFSwabArrayOfShort((unsigned short*)uncompressed_buf, nvals);
         if (m_predictor == PREDICTOR_HORIZONTAL) {
             if (m_spec.format == TypeUInt8)
                 undo_horizontal_predictor((unsigned char*)uncompressed_buf,
@@ -459,7 +451,7 @@ private:
 
 
 
-// Obligatory material to make this a recognizeable imageio plugin:
+// Obligatory material to make this a recognizable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
 OIIO_EXPORT ImageInput*
@@ -518,7 +510,7 @@ oiio_tiff_set_error_handler()
 
 
 static tsize_t
-readproc(thandle_t handle, tdata_t data, tsize_t size)
+reader_readproc(thandle_t handle, tdata_t data, tsize_t size)
 {
     auto io = static_cast<Filesystem::IOProxy*>(handle);
     // Strutil::print("iop read {} {} @ {}\n",
@@ -532,13 +524,13 @@ readproc(thandle_t handle, tdata_t data, tsize_t size)
 }
 
 static tsize_t
-writeproc(thandle_t, tdata_t, tsize_t size)
+reader_writeproc(thandle_t, tdata_t, tsize_t size)
 {
     return tsize_t(0);
 }
 
 static toff_t
-seekproc(thandle_t handle, toff_t offset, int origin)
+reader_seekproc(thandle_t handle, toff_t offset, int origin)
 {
     auto io = static_cast<Filesystem::IOProxy*>(handle);
     // Strutil::print("iop seek {} {} ({})\n",
@@ -547,7 +539,7 @@ seekproc(thandle_t handle, toff_t offset, int origin)
 }
 
 static int
-closeproc(thandle_t handle)
+reader_closeproc(thandle_t handle)
 {
     // auto io = static_cast<Filesystem::IOProxy*>(handle);
     // if (io && io->opened()) {
@@ -560,7 +552,7 @@ closeproc(thandle_t handle)
 }
 
 static toff_t
-sizeproc(thandle_t handle)
+reader_sizeproc(thandle_t handle)
 {
     auto io = static_cast<Filesystem::IOProxy*>(handle);
     // Strutil::print("iop size\n");
@@ -568,23 +560,18 @@ sizeproc(thandle_t handle)
 }
 
 static int
-mapproc(thandle_t, tdata_t*, toff_t*)
+reader_mapproc(thandle_t, tdata_t*, toff_t*)
 {
     return 0;
 }
 
-static void unmapproc(thandle_t, tdata_t, toff_t) {}
+static void reader_unmapproc(thandle_t, tdata_t, toff_t) {}
 
 
 
-struct CompressionCode {
-    int code;
-    const char* name;
-};
-
 // clang-format off
 
-static CompressionCode tiff_compressions[] = {
+static std::pair<int, const char*>  tiff_input_compressions[] = {
     { COMPRESSION_NONE,          "none" },        // no compression
     { COMPRESSION_LZW,           "lzw" },         // LZW
     { COMPRESSION_ADOBE_DEFLATE, "zip" },         // deflate / zip
@@ -617,7 +604,6 @@ static CompressionCode tiff_compressions[] = {
     { COMPRESSION_T43,           "T43" },         // TIFF/FX T.43 color layered JBIG
     { COMPRESSION_LZMA,          "lzma" },        // LZMA2
 #endif
-    { -1, NULL }
 };
 
 // clang-format on
@@ -625,10 +611,10 @@ static CompressionCode tiff_compressions[] = {
 static const char*
 tiff_compression_name(int code)
 {
-    for (int i = 0; tiff_compressions[i].name; ++i)
-        if (code == tiff_compressions[i].code)
-            return tiff_compressions[i].name;
-    return NULL;
+    for (const auto& c : tiff_input_compressions)
+        if (c.first == code)
+            return c.second;
+    return nullptr;
 }
 
 
@@ -701,12 +687,8 @@ bool
 TIFFInput::open(const std::string& name, ImageSpec& newspec,
                 const ImageSpec& config)
 {
-    const ParamValue* param = config.find_attribute("oiio:ioproxy",
-                                                    TypeDesc::PTR);
-    if (param)
-        m_io = param->get<Filesystem::IOProxy*>();
-
     // Check 'config' for any special requests
+    ioproxy_retrieve_from_config(config);
     if (config.get_int_attribute("oiio:UnassociatedAlpha", 0) == 1)
         m_keep_unassociated_alpha = true;
     if (config.get_int_attribute("oiio:RawColor", 0) == 1)
@@ -748,14 +730,16 @@ TIFFInput::seek_subimage(int subimage, int miplevel)
     bool read_meta = !(m_emulate_mipmap && m_tif && m_subimage >= 0);
 
     if (!m_tif) {
-        if (m_io) {
+        if (ioproxy_opened()) {
             static_assert(sizeof(thandle_t) == sizeof(void*),
                           "thandle_t must be same size as void*");
             // Strutil::print("\n\nOpening client \"{}\"\n", m_filename);
-            m_io->seek(0);
-            m_tif = TIFFClientOpen(m_filename.c_str(), "rm", m_io, readproc,
-                                   writeproc, seekproc, closeproc, sizeproc,
-                                   mapproc, unmapproc);
+            ioseek(0);
+            m_tif = TIFFClientOpen(m_filename.c_str(), "rm", ioproxy(),
+                                   reader_readproc, reader_writeproc,
+                                   reader_seekproc, reader_closeproc,
+                                   reader_sizeproc, reader_mapproc,
+                                   reader_unmapproc);
         } else {
 #ifdef _WIN32
             std::wstring wfilename = Strutil::utf8_to_utf16(m_filename);
@@ -948,7 +932,7 @@ TIFFInput::readspec(bool read_meta)
                 && Strutil::parse_char(software, '.')
                 && Strutil::parse_int(software, minor)
                 && Strutil::parse_char(software, '.')
-                && Strutil::parse_int(software, patch)) {
+                && Strutil::parse_int(software, patch)) {  // NOSONAR
                 oiio_write_version = major * 10000 + minor * 100 + patch;
             }
         }
@@ -967,7 +951,7 @@ TIFFInput::readspec(bool read_meta)
 
     // Start by assuming the "full" (aka display) window is the same as the
     // data window. That's what we'll stick to if there is no further
-    // information in the file. But if the file has tags for hte "full"
+    // information in the file. But if the file has tags for the "full"
     // size, assume a display window with origin (0,0) and those dimensions.
     // (Unfortunately, there are no TIFF tags for "full" origin.)
     m_spec.full_x      = m_spec.x;
@@ -1233,13 +1217,23 @@ TIFFInput::readspec(bool read_meta)
     // Search for IPTC metadata in IIM form -- but older versions of
     // libtiff botch the size, so ignore it for very old libtiff.
     int iptcsize         = 0;
-    const void* iptcdata = NULL;
-    if (TIFFGetField(m_tif, TIFFTAG_RICHTIFFIPTC, &iptcsize, &iptcdata)) {
-        std::vector<uint32_t> iptc((uint32_t*)iptcdata,
-                                   (uint32_t*)iptcdata + iptcsize);
-        if (TIFFIsByteSwapped(m_tif))
-            TIFFSwabArrayOfLong((uint32_t*)&iptc[0], iptcsize);
-        decode_iptc_iim(&iptc[0], iptcsize * 4, m_spec);
+    const char* iptcdata = nullptr;
+    TypeDesc iptctype    = tiffgetfieldtype(TIFFTAG_RICHTIFFIPTC);
+    if (TIFFGetField(m_tif, TIFFTAG_RICHTIFFIPTC, &iptcsize, &iptcdata)
+        && iptcsize > 0) {
+        std::vector<char> iptc;
+        if (iptctype.size() == 4) {
+            // Some TIFF files in the wild inexplicably think their IPTC
+            // data are stored as longs, and we have to undo any byte
+            // swapping that may have occurred.
+            iptcsize *= 4;
+            iptc.assign(iptcdata, iptcdata + iptcsize);
+            if (TIFFIsByteSwapped(m_tif))
+                TIFFSwabArrayOfLong((uint32_t*)&iptc[0], iptcsize / 4);
+        } else {
+            iptc.assign(iptcdata, iptcdata + iptcsize);
+        }
+        decode_iptc_iim(&iptc[0], iptcsize, m_spec);
     }
 #endif
 
@@ -1298,6 +1292,12 @@ TIFFInput::readspec(bool read_meta)
         m_spec.attribute("oiio:SHA-1", sha);
         updatedDesc = true;
     }
+    std::string handed = Strutil::excise_string_after_head(desc,
+                                                           "oiio:handed=");
+    if (handed.size() && (handed == "left" || handed == "right")) {
+        m_spec.attribute("handed", handed);
+        updatedDesc = true;
+    }
 
     if (updatedDesc) {
         string_view d(desc);
@@ -1593,9 +1593,10 @@ TIFFInput::read_native_scanline(int subimage, int miplevel, int y, int /*z*/,
         // libtiff has no way to read just one scanline as RGBA. So we
         // buffer the whole image.
         if (!m_rgbadata.size()) {  // first time through: allocate & read
-            m_rgbadata.resize(m_spec.width * m_spec.height * m_spec.depth);
+            m_rgbadata.resize(m_spec.image_pixels());
             bool ok = TIFFReadRGBAImageOriented(m_tif, m_spec.width,
-                                                m_spec.height, &m_rgbadata[0],
+                                                m_spec.height,
+                                                m_rgbadata.data(),
                                                 ORIENTATION_TOPLEFT, 0);
             if (!ok) {
                 errorf("Unknown error trying to read TIFF as RGBA");
@@ -1603,7 +1604,7 @@ TIFFInput::read_native_scanline(int subimage, int miplevel, int y, int /*z*/,
             }
         }
         copy_image(m_spec.nchannels, m_spec.width, 1, 1,
-                   &m_rgbadata[y * m_spec.width], m_spec.nchannels, 4,
+                   &m_rgbadata[y * size_t(m_spec.width)], m_spec.nchannels, 4,
                    4 * m_spec.width, AutoStride, data, m_spec.nchannels,
                    m_spec.width * m_spec.nchannels, AutoStride);
         return true;
@@ -1679,8 +1680,8 @@ TIFFInput::read_native_scanline(int subimage, int miplevel, int y, int /*z*/,
     }
     // Not palette...
 
-    int plane_bytes = m_spec.width * m_spec.format.size();
-    int input_bytes = plane_bytes * m_inputchannels;
+    size_t plane_bytes = m_spec.width * m_spec.format.size();
+    size_t input_bytes = plane_bytes * m_inputchannels;
     // Where to read?  Directly into user data if no channel shuffling, bit
     // shifting, or CMYK conversion is needed, otherwise into scratch space.
     unsigned char* readbuf = (unsigned char*)data;
@@ -1837,7 +1838,7 @@ TIFFInput::read_native_scanlines(int subimage, int miplevel, int ybegin,
         // and more than one, or no point parallelizing
         nstrips > 1
         // only if we are reading scanlines in order
-        && ybegin == m_next_scanline
+        && ybegin == (m_next_scanline + m_spec.y)
         // only if we're threading and don't enter the thread pool recursively!
         && pool->size() > 1
         && !pool->is_worker()
@@ -1943,7 +1944,7 @@ TIFFInput::read_native_scanlines(int subimage, int miplevel, int ybegin,
     }
 
     // If we have left over scanlines, read them serially
-    m_next_scanline = y;
+    m_next_scanline = y - m_spec.y;
     for (; y < yend; ++y) {
         bool ok = read_native_scanline(subimage, miplevel, y, z, data);
         if (!ok)
@@ -1991,7 +1992,8 @@ TIFFInput::read_native_tile(int subimage, int miplevel, int x, int y, int z,
                    &m_rgbadata[vert_offset * m_spec.tile_width
                                + (th - 1) * m_spec.tile_width],
                    m_spec.nchannels, 4, -m_spec.tile_width * 4, AutoStride,
-                   data, m_spec.nchannels, m_spec.nchannels * m_spec.tile_width,
+                   data, m_spec.nchannels,
+                   m_spec.nchannels * imagesize_t(m_spec.tile_width),
                    AutoStride);
         return true;
     }
@@ -2121,7 +2123,7 @@ TIFFInput::read_native_tiles(int subimage, int miplevel, int xbegin, int xend,
                                     OIIO::get_int_attribute("tiff:multithread"));
 
     // If we're not parallelizing, just call the parent class default
-    // implementaiton of read_native_tiles, which will loop over the tiles
+    // implementation of read_native_tiles, which will loop over the tiles
     // and read each one individually.
     if (!parallelize) {
         return ImageInput::read_native_tiles(subimage, miplevel, xbegin, xend,
