@@ -39,34 +39,28 @@ static int MIN_SCANLINES_OR_TILES_PER_CHECKPOINT  = 64;
 class TIFFOutput final : public ImageOutput {
 public:
     TIFFOutput();
-    virtual ~TIFFOutput();
-    virtual const char* format_name(void) const override { return "tiff"; }
-    virtual int supports(string_view feature) const override;
-    virtual bool open(const std::string& name, const ImageSpec& spec,
-                      OpenMode mode = Create) override;
-    virtual bool close() override;
-    virtual bool write_scanline(int y, int z, TypeDesc format, const void* data,
-                                stride_t xstride) override;
-    virtual bool write_scanlines(int ybegin, int yend, int z, TypeDesc format,
-                                 const void* data, stride_t xstride,
-                                 stride_t ystride) override;
-    virtual bool write_tile(int x, int y, int z, TypeDesc format,
-                            const void* data, stride_t xstride,
-                            stride_t ystride, stride_t zstride) override;
-    virtual bool write_tiles(int xbegin, int xend, int ybegin, int yend,
-                             int zbegin, int zend, TypeDesc format,
-                             const void* data, stride_t xstride = AutoStride,
-                             stride_t ystride = AutoStride,
-                             stride_t zstride = AutoStride) override;
-    virtual bool set_ioproxy(Filesystem::IOProxy* ioproxy) override
-    {
-        m_io = ioproxy;
-        return true;
-    }
+    ~TIFFOutput() override;
+    const char* format_name(void) const override { return "tiff"; }
+    int supports(string_view feature) const override;
+    bool open(const std::string& name, const ImageSpec& spec,
+              OpenMode mode = Create) override;
+    bool close() override;
+    bool write_scanline(int y, int z, TypeDesc format, const void* data,
+                        stride_t xstride) override;
+    bool write_scanlines(int ybegin, int yend, int z, TypeDesc format,
+                         const void* data, stride_t xstride,
+                         stride_t ystride) override;
+    bool write_tile(int x, int y, int z, TypeDesc format, const void* data,
+                    stride_t xstride, stride_t ystride,
+                    stride_t zstride) override;
+    bool write_tiles(int xbegin, int xend, int ybegin, int yend, int zbegin,
+                     int zend, TypeDesc format, const void* data,
+                     stride_t xstride = AutoStride,
+                     stride_t ystride = AutoStride,
+                     stride_t zstride = AutoStride) override;
 
 private:
-    TIFF* m_tif               = nullptr;
-    Filesystem::IOProxy* m_io = nullptr;
+    TIFF* m_tif = nullptr;
     std::vector<unsigned char> m_scratch;
     Timer m_checkpointTimer;
     int m_checkpointItems;
@@ -88,7 +82,6 @@ private:
     void init(void)
     {
         m_tif                 = nullptr;
-        m_io                  = nullptr;
         m_checkpointItems     = 0;
         m_compression         = COMPRESSION_ADOBE_DEFLATE;
         m_predictor           = PREDICTOR_NONE;
@@ -98,6 +91,7 @@ private:
         m_outputchans         = 0;
         m_convert_rgb_to_cmyk = false;
         m_bigtiff             = false;
+        ioproxy_clear();
     }
 
     // Close m_tif if it's open, but keep all other internal state, don't do a
@@ -190,7 +184,7 @@ private:
 
 
 
-// Obligatory material to make this a recognizeable imageio plugin:
+// Obligatory material to make this a recognizable imageio plugin:
 OIIO_PLUGIN_EXPORTS_BEGIN
 
 OIIO_EXPORT ImageOutput*
@@ -224,15 +218,10 @@ oiio_tiff_set_error_handler();
 
 
 
-struct CompressionCode {
-    int code;
-    const char* name;
-};
-
 // We comment out a lot of these because we only support a subset for
 // writing. These can be uncommented on a case by case basis as they are
 // thoroughly tested and included in the testsuite.
-static CompressionCode tiff_compressions[] = {
+static std::pair<int, const char*> tiff_output_compressions[] = {
     { COMPRESSION_NONE, "none" },          // no compression
     { COMPRESSION_LZW, "lzw" },            // LZW
     { COMPRESSION_ADOBE_DEFLATE, "zip" },  // deflate / zip
@@ -265,15 +254,14 @@ static CompressionCode tiff_compressions[] = {
 //  { COMPRESSION_T43,           "T43" },         // TIFF/FX T.43 color layered JBIG
 //  { COMPRESSION_LZMA,          "lzma" },        // LZMA2
 #endif
-    { -1, NULL }
 };
 
 static int
 tiff_compression_code(string_view name)
 {
-    for (int i = 0; tiff_compressions[i].name; ++i)
-        if (Strutil::iequals(name, tiff_compressions[i].name))
-            return tiff_compressions[i].code;
+    for (const auto& c : tiff_output_compressions)
+        if (Strutil::iequals(name, c.second))
+            return c.first;
     return COMPRESSION_ADOBE_DEFLATE;  // default
 }
 
@@ -340,17 +328,17 @@ allval(const std::vector<T>& d, T v = T(0))
 }
 
 
-static tsize_t readproc(thandle_t, tdata_t, tsize_t) { return 0; }
+static tsize_t writer_readproc(thandle_t, tdata_t, tsize_t) { return 0; }
 
 static tsize_t
-writeproc(thandle_t handle, tdata_t data, tsize_t size)
+writer_writeproc(thandle_t handle, tdata_t data, tsize_t size)
 {
     auto io = static_cast<Filesystem::IOProxy*>(handle);
     return io->write(data, size);
 }
 
 static toff_t
-seekproc(thandle_t handle, toff_t offset, int origin)
+writer_seekproc(thandle_t handle, toff_t offset, int origin)
 {
     auto io = static_cast<Filesystem::IOProxy*>(handle);
     // Strutil::print("iop seek {} {} ({})\n",
@@ -359,7 +347,7 @@ seekproc(thandle_t handle, toff_t offset, int origin)
 }
 
 static int
-closeproc(thandle_t handle)
+writer_closeproc(thandle_t handle)
 {
     // auto io = static_cast<Filesystem::IOProxy*>(handle);
     // if (io && io->opened()) {
@@ -372,7 +360,7 @@ closeproc(thandle_t handle)
 }
 
 static toff_t
-sizeproc(thandle_t handle)
+writer_sizeproc(thandle_t handle)
 {
     auto io = static_cast<Filesystem::IOProxy*>(handle);
     // Strutil::print("iop size\n");
@@ -380,12 +368,12 @@ sizeproc(thandle_t handle)
 }
 
 static int
-mapproc(thandle_t, tdata_t*, toff_t*)
+writer_mapproc(thandle_t, tdata_t*, toff_t*)
 {
     return 0;
 }
 
-static void unmapproc(thandle_t, tdata_t, toff_t) {}
+static void writer_unmapproc(thandle_t, tdata_t, toff_t) {}
 
 
 
@@ -426,12 +414,7 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
             return false;
         }
     }
-    {
-        const ParamValue* param = m_spec.find_attribute("oiio:ioproxy",
-                                                        TypeDesc::PTR);
-        if (param)
-            m_io = param->get<Filesystem::IOProxy*>();
-    }
+    ioproxy_retrieve_from_config(m_spec);
 
     // Classic/standard TIFF files use 32 bit offsets, and so are limited to
     // a total file size of 4GB. If the image size is getting close to this
@@ -450,17 +433,19 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
     // 3. It probably won't do the right thing for a multi-subimage file,
     //    since (like with compression) it can't possibly know the total
     //    file size upon first opening it.
-    m_bigtiff |= m_spec.get_int_attribute("tiff:bigtiff");
+    m_bigtiff |= m_spec.get_int_attribute("tiff:bigtiff") != 0;
     m_bigtiff |= m_spec.image_bytes() > (4000LL * 1024LL * 1024LL);
     const char* openmode = m_bigtiff ? (mode == AppendSubimage ? "a8" : "w8")
                                      : (mode == AppendSubimage ? "a" : "w");
 
     // Open the file
-    if (m_io) {
-        m_io->seek(0);
-        m_tif = TIFFClientOpen(name.c_str(), openmode, m_io, readproc,
-                               writeproc, seekproc, closeproc, sizeproc,
-                               mapproc, unmapproc);
+    if (ioproxy_opened()) {
+        ioseek(0);
+        m_tif = TIFFClientOpen(name.c_str(), openmode, ioproxy(),
+                               writer_readproc, writer_writeproc,
+                               writer_seekproc, writer_closeproc,
+                               writer_sizeproc, writer_mapproc,
+                               writer_unmapproc);
     } else {
 #ifdef _WIN32
         m_tif = TIFFOpenW(Strutil::utf8_to_utf16(name).c_str(), openmode);
@@ -682,7 +667,8 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
 
     // ExtraSamples tag
     if ((m_spec.alpha_channel >= 0 || m_spec.nchannels > 3)
-        && m_photometric != PHOTOMETRIC_SEPARATED) {
+        && m_photometric != PHOTOMETRIC_SEPARATED
+        && m_spec.get_int_attribute("tiff:write_extrasamples", 1)) {
         bool unass = m_spec.get_int_attribute("oiio:UnassociatedAlpha", 0);
         int defaultchans = m_spec.nchannels >= 3 ? 3 : 1;
         short e          = m_spec.nchannels - defaultchans;
@@ -793,11 +779,16 @@ TIFFOutput::open(const std::string& name, const ImageSpec& userspec,
                       m_spec.extra_attribs[p].type(),
                       m_spec.extra_attribs[p].data());
 
-    std::vector<char> iptc;
-    encode_iptc_iim(m_spec, iptc);
-    if (iptc.size()) {
-        iptc.resize((iptc.size() + 3) & (0xffff - 3));  // round up
-        TIFFSetField(m_tif, TIFFTAG_RICHTIFFIPTC, iptc.size() / 4, &iptc[0]);
+    if (m_spec.get_int_attribute("tiff:write_iptc")) {
+        // Enable IPTC block writing only if "tiff_write_iptc" hint is explicitly
+        // enabled. This was to avoid writing bad IPTC blocks, but I think that
+        // was because of a size mixup on our part that has since been fixed.
+        // Should we re-enable it by default in the future?
+        std::vector<char> iptc;
+        encode_iptc_iim(m_spec, iptc);
+        if (iptc.size()) {
+            TIFFSetField(m_tif, TIFFTAG_RICHTIFFIPTC, iptc.size(), &iptc[0]);
+        }
     }
 
     std::string xmp = encode_xmp(m_spec, true);
@@ -822,6 +813,10 @@ bool
 TIFFOutput::put_parameter(const std::string& name, TypeDesc type,
                           const void* data)
 {
+    if (!data || (type == TypeString && *(char**)data == nullptr)) {
+        // we got a null pointer, don't set the field
+        return false;
+    }
     if (Strutil::iequals(name, "Artist") && type == TypeDesc::STRING) {
         TIFFSetField(m_tif, TIFFTAG_ARTIST, *(char**)data);
         return true;
@@ -1117,7 +1112,7 @@ TIFFOutput::write_scanline(int y, int z, TypeDesc format, const void* data,
     std::vector<unsigned char> cmyk;
     if (m_photometric == PHOTOMETRIC_SEPARATED && m_convert_rgb_to_cmyk)
         data = convert_to_cmyk(spec().width, data, cmyk);
-    size_t scanline_vals = spec().width * m_outputchans;
+    size_t scanline_vals = size_t(spec().width) * m_outputchans;
 
     // Handle weird bit depths
     if (spec().format.size() * 8 != m_bitspersample) {
@@ -1241,7 +1236,7 @@ TIFFOutput::write_scanlines(int ybegin, int yend, int z, TypeDesc format,
                                     OIIO::get_int_attribute("tiff:multithread"));
 
     // If we're not parallelizing, just call the parent class default
-    // implementaiton of write_scanlines, which will loop over the scanlines
+    // implementation of write_scanlines, which will loop over the scanlines
     // and write each one individually.
     if (!parallelize) {
         return ImageOutput::write_scanlines(ybegin, yend, z, format, data,
@@ -1469,7 +1464,7 @@ TIFFOutput::write_tiles(int xbegin, int xend, int ybegin, int yend, int zbegin,
                                     OIIO::get_int_attribute("tiff:multithread"));
 
     // If we're not parallelizing, just call the parent class default
-    // implementaiton of write_tiles, which will loop over the tiles and
+    // implementation of write_tiles, which will loop over the tiles and
     // write each one individually.
     if (!parallelize) {
         return ImageOutput::write_tiles(xbegin, xend, ybegin, yend, zbegin,
